
with Support;                           use Support;

package body Allegro.Truecolor is

    function Compare( a, b : Color_Type; tolerance : Natural := 0 ) return Boolean is
    begin
        if a = b then
            return True;
        end if;

        return abs (GetR( a ) - GetR( b )) <= tolerance and then
               abs (GetG( a ) - GetG( b )) <= tolerance and then
               abs (GetB( a ) - GetB( b )) <= tolerance and then
               abs (GetA( a ) - GetA( b )) <= tolerance;
    end Compare;

    ----------------------------------------------------------------------------

    function Contrast( fg, bg : Color_Type; ratio : Float := 1.75 ) return Color_Type is

        ------------------------------------------------------------------------

        function Luminance( c : Color_Type ) return Float is
            function R( v : Integer ) return Float is
            begin
                return Float(Long_Float(Float(v) / 255) ** 2.2);
            end R;
        begin
            return (0.2126 * R( GetR( c ) ) +
                    0.7152 * R( GetG( c ) ) +
                    0.0722 * R( GetB( c ) ) + 0.05) / 1.05;
        end Luminance;

        ------------------------------------------------------------------------

        function Luminance_Adjust( c : Color_Type; val : Float ) return Color_Type is
            function Adjust( v : Integer; c : Float ) return Integer is
            begin
                return Integer(Float'Max(0.0, Float'Min(255.0, Float(v) - 255.0 * val * c)));
            end Adjust;
        begin
            return Makeacol( Adjust( GetR( c ), 0.2126 ),
                             Adjust( GetG( c ), 0.7152 ),
                             Adjust( GetB( c ), 0.0722 ),
                             GetA( c ) );
        end Luminance_Adjust;

        ------------------------------------------------------------------------

        bg_lum : constant Float := Luminance( bg );
        fg_lum : constant Float := Luminance( fg );
    begin
        if Float'Max( bg_lum, fg_lum ) / Float'Min( bg_lum, fg_lum ) > ratio then
            return fg;
        end if;
        return Luminance_Adjust( fg, Float'Min( Float'Max( bg_lum, fg_lum ), 1.0 ) );
    end Contrast;

    ----------------------------------------------------------------------------

    function Hard_Contrast( bg : Color_Type ) return Color_Type is

        function Is_Dark( color : Color_Type ) return Boolean is

            function Luminance( r, g, b : Integer ) return Float is
                weight_red   : constant Float := 0.3;
                weight_green : constant Float := 0.59;
                weight_blue  : constant Float := 0.11;
            begin
                return weight_red * r +
                       weight_green * g +
                       weight_blue * b;
            end Luminance;

        begin
            return Luminance( GetR( color ), GetG( color ), GetB( color ) ) <= 128.0;
        end Is_Dark;

    begin
        if Is_Dark( bg ) then
            return Makeacol( 255, 255, 255, GetA( bg ) );
        else
            return Makeacol( 0, 0, 0, GetA( bg ) );
        end if;
    end Hard_Contrast;

    ----------------------------------------------------------------------------

    function Lighten( color : Color_Type; factor : Float ) return Color_Type is

        function "*"( left : Integer; right : Float ) return Integer is
        begin
            return Integer'Min( Integer(Float(left) * right), 255 );
        end "*";

    begin
        return Makeacol( GetR( color ) * factor,
                         GetG( color ) * factor,
                         GetB( color ) * factor,
                         GetA( color ) );
    end Lighten;

    ----------------------------------------------------------------------------

    function Make_Grey( brightness : Natural ) return Color_Type is
        b : constant Integer := Constrain( brightness, 0, 255 );
    begin
        return Makeacol( b, b, b, 255 );
    end Make_Grey;

end Allegro.Truecolor;
