
with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Exceptions;                    use Ada.Exceptions;
with Ada.Strings;                       use Ada.Strings;
with Ada.Strings.Fixed;                 use Ada.Strings.Fixed;
with Ada.Text_IO;                       use Ada.Text_IO;
with GNAT.OS_Lib;                       use GNAT.OS_Lib;

procedure REVISION is

#if WINDOWS then
        SEP : constant Character := ';';
#else
        SEP : constant Character := ':';
#end if;

    ---------------------------------------------------------------------------

    function Find_On_Path( name : String ) return String is
        path  : String_Access := Getenv( "PATH" );
        first,
        last  : Integer;
    begin
        if name'Length = 0 then
            return "";
        elsif Is_Absolute_Path( name ) then
            return name;
        end if;

        first := path'First;
        last := path'First;

        while last <= path'Last + 1 loop
            if last = path'Last + 1 or else path(last) = SEP then
                declare
                    pth : constant String := Normalize_Pathname( path(first..last-1) &
                                                                Directory_Separator & name );
                begin
                    if Is_Regular_File( pth ) then
                        Free( path );
                        return pth;
                    end if;
                end;
                first := last + 1;
            end if;
            last := last + 1;
        end loop;

        Free( path );
        return name;
    end Find_On_Path;

    ---------------------------------------------------------------------------

    fd         : File_Descriptor;
    returnCode : Integer;
    rev        : Integer := 0;
    tmpName    : Temp_File_Name;
    success    : Boolean;
    len        : Integer;
    result     : String_Access;
begin
    if Argument_Count = 1 then

        Create_Temp_File( fd, tmpName );
        declare
            args : Argument_List(1..2);
        begin
            args(1) := new String'("-n");
            args(2) := new String'(Argument( 1 ));
            Spawn( Find_On_Path( "svnversion" ), args, fd, returnCode );
            Free( args(1) );
            Free( args(2) );
        end;
        Close( fd );

        fd := Open_Read( tmpName, Text );
        if fd /= Invalid_FD then
            len := Integer(File_Length( fd ));
            if len > 0 then
                result := new String(1..len);
                len := Read( fd, result(result'First)'Address, len );
                if returnCode = 0 then
                    declare
                        revString : String := result(1..len);
                        first     : Integer := Integer'Max( revString'First,
                                                            Index( revString, ":" ) + 1 );
                        last      : Integer := revString'Last;
                    begin
                        for i in first..revString'Last loop
                            if not Is_Digit( revString(i) ) then
                                last := i - 1;
                                exit;
                            end if;
                        end loop;

                        if last < first then
                            raise Constraint_Error;
                        else
                            rev := Integer'Value( revString(first..last) );
                        end if;
                    exception
                        when others =>
                            Put_Line( "revision: Error parsing '" & revString & "'" );
                            Free( result );
                            Close( fd );
                            Delete_File( tmpName, success );
                            OS_Exit( 1 );
                    end;
                else
                    Put_Line( "revision: Error executing svnversion (returned" & returnCode'Img & ")" );
                    Put_Line( result.all );
                    Free( result );
                    Close( fd );
                    Delete_File( tmpName, success );
                    OS_Exit( 1 );
                end if;
                Free( result );
            else
                Put_Line( "revision: Failed to execute svnversion" );
                Close( fd );
                Delete_File( tmpName, success );
                OS_Exit( 1 );
            end if;
            Close( fd );
        else
            Put_Line( "revision: Failed to read output from svnversion" );
            Delete_File( tmpName, success );
            OS_Exit( 1 );
        end if;

        Delete_File( tmpName, success );
    else
        Put_Line( "revision: Incorrect number of arguments" );
        OS_Exit( 1 );
    end if;

    Put( Trim( Integer'Image( rev ), Left ) );
exception
    when e : others =>
        Put_Line( "revision: Exception: " & Exception_Name( e ) &
                  " -- " & Exception_Message( e ) );
        OS_Exit( 1 );
end REVISION;
