
with Ada.Exceptions;                    use Ada.Exceptions;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Games;                             use Games;

package body Actors is

    procedure Initialize is
    begin
        Factory.Initialize( "actor" );
    end Initialize;

    ----------------------------------------------------------------------------

    procedure Finalize is
    begin
        Factory.Finalize;
    end Finalize;

    --==========================================================================

    function Create_Actor( entityClass : String ) return A_Actor is
    begin
        return Factory.Allocate( entityClass );
    exception
        when e : others =>
            pragma Debug( Dbg( "Error creating actor class <" & entityClass &
                               ">: " & Exception_Name( e ) & " - " &
                               Exception_Message( e ),
                               D_ENTITIES, ERROR ) );
            return null;
    end Create_Actor;

    ----------------------------------------------------------------------------

    not overriding
    function Is_Temporal( this : access Actor ) return Boolean is
        pragma Unreferenced( this );
    begin
        return True;
    end Is_Temporal;

    ----------------------------------------------------------------------------

    procedure Set_Game( this : not null access Actor'Class;
                        game : not null access Games.Game'Class ) is
    begin
        this.game := game;
    end Set_Game;

    ----------------------------------------------------------------------------

    not overriding
    function Set_Entity( this : access Actor; e : A_Entity ) return Boolean is
        pragma Unreferenced( this, e );
    begin
        return False;
    end Set_Entity;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Entity( this : access Actor; e : A_Entity ) is
    begin
        this.entity := e;
    end Set_Entity;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Actor ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Actors;
