
with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Entities.Players;                  use Entities.Players;
with Resources;                         use Resources;
with Tiles.Libraries;                   use Tiles.Libraries;

package body Entities.Triggers is

    DEFAULT_WIDTH  : constant := 32;
    DEFAULT_HEIGHT : constant := 32;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this    : access Trigger;
                         libName : String;
                         icon    : String ) is
    begin
        Entity(this.all).Construct( DEFAULT_WIDTH, DEFAULT_HEIGHT, libName );
        this.frame := this.lib.Get_Id( icon );
        if this.frame = 0 then
            raise RESOURCE_ERROR with "Image <" & icon & "> not found in " &
                                      "tile library <" & libName & ">";
        end if;
        this.metaphysical := True;
        this.physical := False;
        this.clipped := False;
        this.attrs.Set_Value( ACTION_ATTR, "" );
        this.attrs.Set_Value( ARG_ATTR, "" );
        this.attrs.Set_Value( WHEN_ATTR, "OnActivate" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Trigger ) is
    begin
        Entity'Read( stream, Entity(obj) );
        obj.activatable := Boolean'Input( stream );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Trigger ) is
    begin
        Entity'Write( stream, Entity(obj) );
        Boolean'Output( stream, obj.activatable );
    end Object_Write;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Activate( this      : access Trigger;
                           activator : not null A_Entity ) is
    begin
        if this.activatable and then
           activator.all in Player'Class and then
           this.attrs.As_String( WHEN_ATTR ) = "OnActivate"
        then
            A_Trigger(this).Execute( To_Lower( this.attrs.As_String( ACTION_ATTR ) ),
                                     To_Lower( this.attrs.As_String( ARG_ATTR ) ),
                                     activator );
        end if;
    end On_Activate;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Collide( this : access Trigger; e : not null A_Entity ) is
    begin
        if this.activatable and then
           e.all in Player'Class and then
           this.attrs.As_String( WHEN_ATTR ) = "OnTouch" and then
           not this.editingMode
        then
            A_Trigger(this).Execute( To_Lower( this.attrs.As_String( ACTION_ATTR ) ),
                                     this.attrs.As_String( ARG_ATTR ),
                                     e );
        end if;
    end On_Collide;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Trigger ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Entities.Triggers;
