
--with Debugging;                         use Debugging;
with Resources;                         use Resources;

package body Font_API.Alfont_Fonts is

    procedure Initialize is
    begin
        if Alfont.Alfont_Init /= ALFONT_OK then
            raise FONT_EXCEPTION with "AllegroFont failed to initialize";
        end if;
    end Initialize;

    ----------------------------------------------------------------------------

    procedure Finalize is
    begin
        Alfont.Alfont_Exit;
    end Finalize;

    --==========================================================================

    function Load_Font( filename : String; size : Positive ) return Font_Type is
        this : Font_Type := new Alfont_Font;
    begin
        Alfont_Font(this.all).Construct( filename, size );
        return this;
    exception
        when others =>
            Delete( this );
            return null;
    end Load_Font;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this     : access Alfont_Font;
                         filename : String;
                         size     : Positive ) is
        resource : A_Resource_File := null;
    begin
        Abstract_Font(this.all).Construct;
        resource := Load_Resource( filename, "resources" );
        if resource /= null then
            this.ptr := Alfont.Load_Font_From_Mem( resource.Get_Address, Integer(resource.Size) );
            Unload_Resource( resource );
            if this.ptr /= null then
                if Alfont.Set_Font_Size( this.ptr, size ) /= ALFONT_OK then
                    raise Constraint_Error with "Error setting font size" & Positive'Image( size );
                end if;
            else
                raise Constraint_Error with "Bad font file <" & filename & '>';
            end if;
        else
            raise RESOURCE_ERROR with "Error loading font <" & filename & '>';
        end if;
    exception
        when others =>
            Unload_Resource( resource );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Alfont_Font ) is
    begin
        Alfont.Destroy_Font( this.ptr );
        Abstract_Font(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Text_Height( this : access Alfont_Font ) return Positive is
    begin
        return Alfont.Text_Height( this.ptr );
    end Text_Height;

    ----------------------------------------------------------------------------

    overriding
    function Text_Length( this : access Alfont_Font;
                          str  : String ) return Natural is
    begin
        return Alfont.Text_Length( this.ptr, str );
    end Text_Length;

    ----------------------------------------------------------------------------

    overriding
    procedure Textout( this   : access Alfont_Font;
                       bmp    : A_Bitmap;
                       str    : String;
                       x, y   : Integer;
                       color  : Color_Type;
                       smooth : Boolean ) is
    begin
        if smooth then
            Alfont.Textout_aa_ex( bmp, this.ptr, str, x, y, color, -1 );
        else
            Alfont.Textout_ex( bmp, this.ptr, str, x, y, color, -1 );
        end if;
    end Textout;

end Font_API.Alfont_Fonts;
