
with Ada.Tags;                          use Ada.Tags;
with Ada.Text_IO;                       use Ada.Text_IO;
with Ada.Unchecked_Deallocation;

package body Pool_Testing is

    not overriding
    procedure Construct( this     : access Car;
                         topSpeed : Integer ) is
    begin
        this.topSpeed := topSpeed;
        Put_Line( "Constructing Car.topSpeed =" & topSpeed'Img );
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    procedure Drive( this : access Car ) is
        pragma Unreferenced( this );
    begin
        Put_Line( "putt putt putt" );
    end Drive;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Car ) is
    begin
        if this /= null then
            this.Destruct;
            this.Destroy;
            this := null;
        end if;
    end Delete;

    ----------------------------------------------------------------------------

    not overriding
    procedure Destruct( this : access Car ) is
        pragma Unreferenced( this );
    begin
        Put_Line( "Deleting Car.topSpeed" );
    end Destruct;

    --==========================================================================

    function Create_Audi return A_Audi is
        this : constant A_Audi := new Audi;
    begin
        Put_Line( "Allocated with pool " & External_Tag( this'Storage_Pool'Tag ) );
        this.Construct( 160, True );
        return this;
    end Create_Audi;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this      : access Audi;
                         topSpeed  : Integer;
                         tipTronic : Boolean ) is
    begin
        this.Construct( topSpeed );
        this.tipTronic := tipTronic;
        Put_Line( "Constructing Audi.tipTronic = " & tipTronic'Img );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Drive( this : access Audi ) is
        pragma Unreferenced( this );
    begin
        Put_Line( "Vroom! Vrooom!!" );
        null;
    end Drive;

    ----------------------------------------------------------------------------

    overriding
    procedure Destruct( this : access Audi ) is
    begin
        Put_Line( "Deleting Audi.tipTronic" );
        Car(this.all).Destruct;
    end Destruct;

    ----------------------------------------------------------------------------

    overriding
    procedure Destroy( this : access Audi ) is
        procedure Free is new Ada.Unchecked_Deallocation( Audi'Class, A_Audi );
        ths : A_Audi := A_Audi(this);
    begin
        Put_Line( "Freeing with pool " & External_Tag( ths'Storage_Pool'Tag ) );
        Free( ths );
    end Destroy;


    --==========================================================================

    procedure Go is
        it : A_Car;
    begin
        it := A_Car(Create_Audi);
        it.Drive;
        Delete( it );
    end Go;

end Pool_Testing;
