
package Preferences is

    -- Initializes the preference system. The prefences in file 'filename' will
    -- be loaded and the auto-save task will begin.
    procedure Initialize( filename : String );

    -- Finalizes the preference system. All non-default preferences will be
    -- written to the filename provided at initialization, and the auto-save
    -- task will end.
    procedure Finalize;

    ----------------------------------------------------------------------------

    -- These procedures return a value for the section.name preference. If the
    -- preference has never been set, a default value will be returned as
    -- available. If 'section' is not specified, "preference" will be used.

    function Get_Pref( name : String ) return Boolean;

    function Get_Pref( name : String ) return Float;

    function Get_Pref( name : String ) return Integer;

    function Get_Pref( name : String ) return String;

    function Get_Pref( section, name : String ) return Boolean;

    function Get_Pref( section, name : String ) return Float;

    function Get_Pref( section, name : String ) return Integer;

    function Get_Pref( section, name : String ) return String;

    ----------------------------------------------------------------------------

    -- These procedures set the value of the section.name preference. All set
    -- values will be saved to the preference file. If 'section' is not
    -- specified, "preference" will be used.

    procedure Set_Pref( name : String; value : Boolean );

    procedure Set_Pref( name : String; value : Float );

    procedure Set_Pref( name : String; value : Integer );

    procedure Set_Pref( name : String; value : String );

    procedure Set_Pref( section, name : String; value : Boolean );

    procedure Set_Pref( section, name : String; value : Float );

    procedure Set_Pref( section, name : String; value : Integer );

    procedure Set_Pref( section, name : String; value : String );

    ----------------------------------------------------------------------------

    -- These procedures set a default value for the section.name preference.
    -- Default values are not written to the preference file. If 'section' is
    -- not specified, "preference" will be used.

    procedure Set_Default( name : String; value : Boolean );

    procedure Set_Default( name : String; value : Float );

    procedure Set_Default( name : String; value : Integer );

    procedure Set_Default( name : String; value : String );

    procedure Set_Default( section, name : String; value : Boolean );

    procedure Set_Default( section, name : String; value : Float );

    procedure Set_Default( section, name : String; value : Integer );

    procedure Set_Default( section, name : String; value : String );

end Preferences;
