
with Allegro.Files;                     use Allegro.Files;

-- This package is a high-level API for streaming to and from standard Allegro
-- packfiles using standard Ada streams.

package Streams.Packfiles is

    type Packfile_Stream is new Root_Stream_Type with private;
    type A_Packfile_Stream is access all Packfile_Stream'Class;

    ----------------------------------------------------------------------------

    -- Opens a packfile on disk for streaming operations.
    function Open_Stream( filename, mode : String ) return A_Packfile_Stream;

    -- Creates a stream using an open packfile.
    function Stream( f : not null A_Packfile ) return A_Packfile_Stream;

    -- Closes the stream and its internal packfile if it opened one.
    procedure Close( stream : in out A_Packfile_Stream );

    -- Returns True of the stream has reached end-of-file.
    function EOF( stream : access Packfile_Stream ) return Boolean;

    -- Returns True if the stream encountered an error.
    function Error( stream : access Packfile_Stream ) return Boolean;

private

    type Packfile_Stream is new Root_Stream_Type with record
        packfile : A_Packfile := null;
        doClose  : Boolean := False;  -- True if the stream's underlying
                                      -- packfile was opened by the packfile
                                      -- stream (Open_Stream) and should be
                                      -- closed when the stream is closed.
    end record;

    procedure Read( stream : in out Packfile_Stream;
                    item   : out Stream_Element_Array;
                    last   : out Stream_Element_Offset );

    procedure Write( stream : in out Packfile_Stream;
                     item   : Stream_Element_Array );

end Streams.Packfiles;
