
with Applications;                      use Applications;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Themes.Palette;                    use Themes.Palette;

package body Widgets.Buttons is

    not overriding
    procedure Add_Listener( this     : access Button;
                            listener : not null A_Button_Listener ) is
    begin
        this.Add_Listener( "Button_Listener", A_Action_Listener(listener) );
    end Add_Listener;

    ----------------------------------------------------------------------------

    not overriding
    procedure Add_Listener( this    : access Button;
                            handler : not null A_Button_Handler ) is
        listener : constant A_Button_Listener := Create_Listener( handler );
    begin
        this.Add_Listener( listener );
    end Add_Listener;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Button;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String;
                         text : String;
                         icon : String ) is
    begin
        Widget(this.all).Construct( view, id );
        this.border := Border_Raised;
        this.Set_Text( text );
        this.Set_Icon( icon );
    end Construct;

    ----------------------------------------------------------------------------

    procedure Contrast_Text( this : not null access Button'Class; enabled : Boolean ) is
    begin
        if this.contrastText /= enabled then
            this.contrastText := enabled;
            A_Widget(this).Set_Dirty;
        end if;
    end Contrast_Text;

    ----------------------------------------------------------------------------

    not overriding
    procedure Dispatch_Action( this : access Button; id : Action_Id ) is
        action : A_Button_Action := new Button_Action;

        procedure Dispatch( listener : A_Action_Listener ) is
        begin
            A_Button_Listener(listener).Handle_Action( action );
        end Dispatch;

    begin
        action.Construct( id, A_Widget(this) );
        this.Iterate_Listeners( "Button_Listener", Dispatch'Access );
        Delete( action );
    end Dispatch_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Button; dc : Drawing_Context ) is
        theme     : constant A_Theme := Get_Application.Get_Theme;
        icon      : constant A_Bitmap := theme.Get_Bitmap( this.icon );
        border    : Border_Type := this.border;
        bgColor   : Color_Type;
        textColor : Color_Type := this.colors(Text);
        bw        : constant Natural := theme.Border_Width;
        pad       : Natural := 0;
    begin
        if not this.enabled then
            bgColor := Lighten( this.colors(Foreground), 0.90 );
        elsif this.on then
            bgColor := Lighten( this.colors(Foreground), 0.85 );
            if border = Border_Raised then
                border := Border_Sunk;
            end if;
        else
            bgColor := this.colors(Foreground);
            if this.hover then
                bgColor := Lighten( bgColor, 1.1 );
            end if;
        end if;

        if this.contrastText then
            textColor := Hard_Contrast( bgColor );
        end if;

        if not this.transparent then
            theme.Draw_Box( dc, 0, 0, this.cwidth - 1, this.cheight - 1, bgColor, border );
        end if;

        if this.focused and then this.border /= Border_None then
            Rect( dc,
                  bw, bw, this.cwidth - 1 - bw, this.cheight - 1 - bw,
                  this.colors(Selected) );
        end if;
        if this.border /= Border_None then
            pad := bw + SPACING;
        end if;
        theme.Draw_Label( dc,
                          pad, pad,
                          this.cwidth - 1 - pad,
                          this.cheight - 1 - pad,
                          icon, To_String( this.text ),
                          To_String( this.fontName ), this.fontSize,
                          textColor, this.align, smooth => this.fontSmooth );
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Height( this : access Button ) return Natural is
        theme : A_Theme;
        icon  : A_Bitmap;
        pad   : Natural := 0;
    begin
        if this.minHeight > 0 then
            return this.minHeight;
        else
            theme := Get_Application.Get_Theme;
            icon := theme.Get_Bitmap( this.icon );
            if this.border /= Border_None then
                pad := theme.Border_Width * 2;
            end if;
            return pad + theme.Label_Height( icon, To_String( this.text ),
                                             To_String( this.fontName ), this.fontSize );
        end if;
    end Get_Min_Height;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Width( this : access Button ) return Natural is
        theme : A_Theme;
        icon  : A_Bitmap;
        pad   : Natural := 0;
    begin
        if this.minWidth > 0 then
            return this.minWidth;
        else
            theme := Get_Application.Get_Theme;
            icon := theme.Get_Bitmap( this.icon );
            if this.border /= Border_None then
                pad := theme.Border_Width * 2;
            end if;
            return pad + theme.Label_Width( icon, To_String( this.text ),
                                            To_String( this.fontName ), this.fontSize );
        end if;
    end Get_Min_Width;

    ----------------------------------------------------------------------------

    function Get_State( this : not null access Button'Class ) return Boolean is
    begin
        return this.on;
    end Get_State;

    ----------------------------------------------------------------------------

    function Get_Text( this : not null access Button'Class ) return String is
    begin
        return To_String( this.text );
    end Get_Text;

    ----------------------------------------------------------------------------

    not overriding
    procedure Remove_Listener( this     : access Button;
                               listener : not null A_Button_Listener ) is
    begin
        this.Remove_Listener( "Button_Listener", A_Action_Listener(listener) );
    end Remove_Listener;

    ----------------------------------------------------------------------------

    procedure Set_Align( this : not null access Button'Class; align : Align_Type ) is
    begin
        if this.align /= align then
            this.align := align;
            A_Widget(this).Set_Dirty;
        end if;
    end Set_Align;

    ----------------------------------------------------------------------------

    overriding
    procedure Set_Color( this    : access Button;
                         purpose : Color_Purpose;
                         color   : Color_Type ) is
    begin
        Widget(this.all).Set_Color( purpose, color );
        this.transparent := purpose = Foreground and then
                            color = base_colors(Transparent);
    end Set_Color;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Icon( this : access Button; icon : String ) is
    begin
        if icon'Length > 0 then
            this.icon := Get_Application.Get_Theme.Get_ID( icon );
            pragma Debug( this.icon = 0,
                          Dbg( "Warning: Icon <" & icon & "> not found for widget " &
                               To_String( this ),
                               D_GUI, WARNING ) );
        else
            this.icon := 0;
        end if;
        A_Widget(this).Set_Dirty;
    end Set_Icon;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_State( this : access Button; on : Boolean ) is
    begin
        if this.on /= on then
            this.on := on;
            A_Widget(this).Set_Dirty;
            if this.on then
                this.Dispatch_Action( Press );
            else
                this.Dispatch_Action( Release );
            end if;
        end if;
    end Set_State;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Text( this : access Button; text : String ) is
    begin
        if text /= To_String( this.text ) then
            this.text := To_Unbounded_String( text );
            A_Widget(this).Set_Dirty;
        end if;
    end Set_Text;

    ----------------------------------------------------------------------------

    not overriding
    procedure Toggle_State( this : access Button ) is
    begin
        A_Button(this).Set_State( not this.on );
    end Toggle_State;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Button ) return String is
    begin
        return "<" & this.Get_Class_Name &
                     " : id=" & To_String( this.id ) &
                     ", text='" & To_String( this.text ) & "'" &
               ">";
    end To_String;

    --==========================================================================

    function Create_Listener( handler : not null A_Button_Handler ) return A_Button_Listener is
        this : constant A_Simple_Button_Listener := new Simple_Button_Listener;
    begin
        this.Construct( handler );
        return A_Button_Listener(this);
    end Create_Listener;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this    : access Simple_Button_Listener;
                         handler : not null A_Button_Handler ) is
    begin
        Simple_Action_Listener(this.all).Construct;
        this.handler := handler;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Simple_Button_Listener;
                             action : A_Button_Action ) is
    begin
        this.handler( action );
    end Handle_Action;

    --==========================================================================

    procedure Delete( this : in out A_Button_Action ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Widgets.Buttons;
