
with Applications;                      use Applications;
--with Debugging;                         use Debugging;
with Scrollables;                       use Scrollables;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;

-- necessary due to compiler bug: 6.2.0w-20081213-43
with Widgets.Containers;                use Widgets.Containers;

package body Widgets.Containers.Scroll_Panes is

    DEFAULT_SCROLL_INCREMENT : constant := 4;
    SCROLLBAR_WIDTH          : constant := 16;

    --==========================================================================

    function Create_Scroll_Pane( view : not null access Game_Views.Game_View'Class;
                                 id   : String ) return A_Scroll_Pane is
        this : A_Scroll_Pane := new Scroll_Pane;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Scroll_Pane;

    ----------------------------------------------------------------------------

    not overriding
    procedure Adjust_Layout( this : access Scroll_Pane ) is
        h2, w2 : Integer;
    begin
        -- set positioning for the scrollbars and buttons
        if this.drawVbar then
            this.up.Set_Layout_LTRB( -SCROLLBAR_WIDTH, 0, 0, -SCROLLBAR_WIDTH );
            this.right.Set_Layout_LTRB( -(SCROLLBAR_WIDTH + 1) * 2, -SCROLLBAR_WIDTH, SCROLLBAR_WIDTH + 1, 0 );
            this.hscroll.Set_Layout_LTRB( SCROLLBAR_WIDTH + 1, -SCROLLBAR_WIDTH, (SCROLLBAR_WIDTH + 1) * 2 + 1, 0 );
        else
            this.right.Set_Layout_LTRB( -SCROLLBAR_WIDTH, -SCROLLBAR_WIDTH, 0, 0 );
            this.hscroll.Set_Layout_LTRB( SCROLLBAR_WIDTH + 1, -SCROLLBAR_WIDTH, SCROLLBAR_WIDTH + 1, 0 );
        end if;
        if this.drawHbar then
            this.left.Set_Layout_LTRB( 0, -SCROLLBAR_WIDTH, -SCROLLBAR_WIDTH, 0 );
            this.down.Set_Layout_LTRB( -SCROLLBAR_WIDTH, -(SCROLLBAR_WIDTH + 1) * 2, 0, SCROLLBAR_WIDTH + 1 );
            this.vscroll.Set_Layout_LTRB( -SCROLLBAR_WIDTH, SCROLLBAR_WIDTH + 1, 0, (SCROLLBAR_WIDTH + 1) * 2 + 1 );
        else
            this.down.Set_Layout_LTRB( -SCROLLBAR_WIDTH, -SCROLLBAR_WIDTH, 0, 0 );
            this.vscroll.Set_Layout_LTRB( -SCROLLBAR_WIDTH, SCROLLBAR_WIDTH + 1, 0, SCROLLBAR_WIDTH + 1 );
        end if;

        -- hide the horizonal scrollbar and adjust left/right if not wide enough
        if this.drawHbar then
            if (this.drawVbar and then this.cwidth - SCROLLBAR_WIDTH <= SCROLLBAR_WIDTH * 2 + 2) or else
               (this.cwidth <= SCROLLBAR_WIDTH * 2 + 2) then
                -- squash out the horizontal scrollbar and adjust the button sizes
                if this.drawVbar then
                    w2 := (this.Get_Width - SCROLLBAR_WIDTH) / 2;
                    this.right.Set_Layout_LTRB( w2 + 1, -SCROLLBAR_WIDTH, SCROLLBAR_WIDTH + 1, 0 );
                else
                    w2 := this.Get_Width / 2;
                    this.right.Set_Layout_LTRB( w2 + 1, -SCROLLBAR_WIDTH, 0, 0 );
                end if;
                this.left.Set_Layout_LTRB( 0, -SCROLLBAR_WIDTH, -w2, 0 );
                this.hscroll.Set_Visible( False );
            else
                this.hscroll.Set_Visible( True );
            end if;
        end if;

        -- hide the vertical scrollbar and adjust up/down if not tall enough
        if this.drawVbar then
            if (this.drawHbar and then this.cheight - SCROLLBAR_WIDTH <= SCROLLBAR_WIDTH * 2 + 2) or else
               (this.cheight <= SCROLLBAR_WIDTH * 2 + 2) then
                -- squash out the vertical scrollbar and adjust the button sizes
                if this.drawHbar then
                    h2 := (this.Get_Height - SCROLLBAR_WIDTH) / 2;
                    this.down.Set_Layout_LTRB( -SCROLLBAR_WIDTH, h2 + 1, 0, SCROLLBAR_WIDTH + 1 );
                else
                    h2 := this.Get_Height / 2;
                    this.down.Set_Layout_LTRB( -SCROLLBAR_WIDTH, h2 + 1, 0, 0 );
                end if;
                this.up.Set_Layout_LTRB( -SCROLLBAR_WIDTH, 0, 0, -h2 );
                this.vscroll.Set_Visible( False );
            else
                this.vscroll.Set_Visible( True );
            end if;
        end if;

        if this.client /= null then
            declare
                top    : constant Integer := this.padding;
                left   : constant Integer := this.padding;
                bottom : Integer := this.padding;
                right  : Integer := this.padding;
            begin
                if this.drawVbar then
                    right := right + (SCROLLBAR_WIDTH + 1);
                end if;
                if this.drawHbar then
                    bottom := bottom + (SCROLLBAR_WIDTH + 1);
                end if;
                this.client.Set_Layout_LTRB( left, top, right, bottom );
            end;
        end if;
        A_Widget(this).Set_Dirty;
    end Adjust_Layout;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Scroll_Pane;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
    begin
        Container(this.all).Construct( view, id );
        this.acceptFocus := False;
        this.border := Border_Raised;

        this.up := Create_Push_Button( this.Get_View, id & ":btnUp", icon => "scroll_up" );
        this.up.Set_Focusable( False );
        this.up.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(this.up), consume => False );

        this.down := Create_Push_Button( this.Get_View, id & ":btnDown", icon => "scroll_down" );
        this.down.Set_Focusable( False );
        this.down.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(this.down), consume => False );

        this.left := Create_Push_Button( this.Get_View, id & ":btnLeft", icon => "scroll_left" );
        this.left.Set_Focusable( False );
        this.left.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(this.left), consume => False );

        this.right := Create_Push_Button( this.Get_View, id & ":btnRight", icon => "scroll_right" );
        this.right.Set_Focusable( False );
        this.right.Add_Listener( A_Button_Listener(this) );
        this.Add( A_Widget(this.right), consume => False );

        this.hscroll := Create_H_Scrollbar( this.Get_View, id & ":hscroll" );
        this.vscroll := Create_V_Scrollbar( this.Get_View, id & ":vscroll" );
        this.Add( A_Widget(this.hscroll), consume => False );
        this.Add( A_Widget(this.vscroll), consume => False );

        this.Adjust_Layout;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Scroll_Pane; dc : Drawing_Context ) is
        theme  : constant A_Theme := Get_Application.Get_Theme;
        top    : constant Integer := 0;
        left   : constant Integer := 0;
        bottom : Integer := this.cheight - 1;
        right  : Integer := this.cwidth - 1;
    begin
        if this.drawVbar then
            right := right - (SCROLLBAR_WIDTH + 1);
        end if;
        if this.drawHbar then
            bottom := bottom - (SCROLLBAR_WIDTH + 1);
        end if;
        theme.Draw_Border( dc, this.border,
                           left, top, right, bottom,
                           this.colors(Foreground) );
        if this.padding > 0 then
            -- +--------+
            -- |________|  top (full width)
            -- |  |##|  |
            -- |  |##|  |  left and right (from top padding to bottom padding)
            -- |__|##|__|
            -- |________|  bottom (full width)

            -- top padding area
            Rectfill( dc,
                      left, top,
                      right, top + this.padding - 1,
                      this.colors(Background) );
            -- bottom padding area
            Rectfill( dc,
                      left, bottom - this.padding + 1,
                      right, bottom,
                      this.colors(Background) );
            -- left padding area
            Rectfill( dc,
                      left, top + this.padding,
                      left + this.padding - 1, bottom - this.padding + 1,
                      this.colors(Background) );
            -- right padding area
            Rectfill( dc,
                      right - this.padding + 1, top + this.padding,
                      right, bottom - this.padding + 1,
                      this.colors(Background) );
        end if;
        if this.drawHbar and then this.drawVbar then
            Rectfill( dc,
                      right + 1, bottom + 1, this.cwidth - 1, this.cheight - 1,
                      this.colors(Background) );
        end if;
    end Draw_Content;

    ----------------------------------------------------------------------------

    not overriding
    procedure Draw_Hbar( this : access Scroll_Pane; draw : Boolean ) is
    begin
        if draw /= this.drawHbar then
            this.drawHbar := draw;
            this.left.Set_Visible( this.drawHbar );
            this.hscroll.Set_Visible( this.drawHbar );
            this.right.Set_Visible( this.drawHbar );
            this.Adjust_Layout;
        end if;
    end Draw_Hbar;

    ----------------------------------------------------------------------------

    not overriding
    procedure Draw_Vbar( this : access Scroll_Pane; draw : Boolean ) is
    begin
        if draw /= this.drawVbar then
            this.drawVbar := draw;
            this.up.Set_Visible( this.drawVbar );
            this.vscroll.Set_Visible( this.drawVbar );
            this.down.Set_Visible( this.drawVbar );
            this.Adjust_Layout;
        end if;
    end Draw_Vbar;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Inc_X( this : access Scroll_Pane ) return Integer is
    begin
        if this.client /= null and then this.client.all in Scrollable'Class then
            return A_Scrollable(this.client).Get_Scroll_Inc_X;
        end if;
        return DEFAULT_SCROLL_INCREMENT;
    end Get_Inc_X;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Inc_Y( this : access Scroll_Pane ) return Integer is
    begin
        if this.client /= null and then this.client.all in Scrollable'Class then
            return A_Scrollable(this.client).Get_Scroll_Inc_Y;
        end if;
        return DEFAULT_SCROLL_INCREMENT;
    end Get_Inc_Y;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Scroll_Pane;
                             action : A_Button_Action ) is
        id : constant Action_Id := action.Get_Id;
    begin
        if (id = Press or else id = Held) and then this.client /= null then
            if action.Get_Source = A_Widget(this.up) then
                this.client.Scroll( 0, -this.Get_Inc_Y );
            elsif action.Get_Source = A_Widget(this.down) then
                this.client.Scroll( 0, this.Get_Inc_Y );
            end if;
            A_Widget(this.vscroll).Set_Dirty;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    function Handle_Mouse_Scroll( this : access Scroll_Pane;
                                  evt  : not null A_Mouse_Scroll_Event ) return Boolean is
        handled : constant Boolean :=
                  this.client /= null and then
                  this.client.cheight > this.client.y2 - this.client.y1 + 1;
    begin
        if this.client /= null then
            this.client.Scroll( 0, evt.Get_Amount * this.Get_Inc_Y );
            A_Widget(this.vscroll).Set_Dirty;
        end if;
        return handled;
    end Handle_Mouse_Scroll;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Resize( this : access Scroll_Pane ) is
    begin
        Container(this.all).Handle_Resize;
        this.Adjust_Layout;
    end Handle_Resize;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Client( this   : access Scroll_Pane;
                          client : in out A_Widget ) is
    begin
        if client /= this.client then
            if this.client /= null then
                this.Remove( this.client );
            end if;
            this.client := client;
            this.Add( client );
            this.Adjust_Layout;
            this.hscroll.Set_Client( this.client );
            this.vscroll.Set_Client( this.client );
        end if;
        client := null;
    end Set_Client;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Padding( this : access Scroll_Pane; padding : Natural ) is
    begin
        if padding /= this.padding then
            this.padding := padding;
            this.Adjust_Layout;
        end if;
    end Set_Padding;

end Widgets.Containers.Scroll_Panes;
