
with System;                            use System;

package hqx is

    pragma Pure;

    -- Enlarges a image by 3x magnification using the hq2x filter. The
    -- destination image must be at least twice as large as the source.
    --
    -- src    : the source image data
    -- dest   : the destination image data (must be large enough!)
    -- srcXres: the width of the source image in pixels
    -- dstXres: the width of the destination image in pixels
    -- width  : the width of the area from the source image to scale
    -- height : the height of the area from the source image to scale
    procedure hq2x( src     : Address;
                    srcXres : Positive;
                    dst     : Address;
                    dstXres : Positive;
                    width,
                    height  : Positive );
    pragma Precondition( src /= Null_Address );
    pragma Precondition( dst /= Null_Address );
    pragma Precondition( width <= srcXres );
    pragma Precondition( dstXres >= width * 2 );

    -- Enlarges an image by 3x magnification using the hq3x filter. The
    -- destination image must be at least three times the size of the source.
    --
    -- src    : the source image data
    -- dest   : the destination image data (must be large enough!)
    -- srcXres: the width of the source image in pixels
    -- dstXres: the width of the destination image in pixels
    -- width  : the width of the area from the source image to scale
    -- height : the height of the area from the source image to scale
    procedure hq3x( src     : Address;
                    srcXres : Positive;
                    dst     : Address;
                    dstXres : Positive;
                    width,
                    height  : Positive );
    pragma Precondition( src /= Null_Address );
    pragma Precondition( dst /= Null_Address );
    pragma Precondition( width <= srcXres );
    pragma Precondition( dstXres >= width * 3 );

    -- Enlarges an image by 4x magnification using the hq4x filter. The
    -- destination image must be at least four times the size of the source.
    --
    -- src    : the source image data
    -- dest   : the destination image data (must be large enough!)
    -- srcXres: the width of the source image in pixels
    -- dstXres: the width of the destination image in pixels
    -- width  : the width of the area from the source image to scale
    -- height : the height of the area from the source image to scale
    procedure hq4x( src     : Address;
                    srcXres : Positive;
                    dst     : Address;
                    dstXres : Positive;
                    width,
                    height  : Positive );
    pragma Precondition( src /= Null_Address );
    pragma Precondition( dst /= Null_Address );
    pragma Precondition( width <= srcXres );
    pragma Precondition( dstXres >= width * 4 );

end hqx;
