
with Actions;                           use Actions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Game_Views.Ked.Actions;
with Keyboard;                          use Keyboard;
with Preferences;                       use Preferences;
with Widgets.Containers;                use Widgets.Containers;
with Widgets.Containers.Panels.Dialogs; use Widgets.Containers.Panels.Dialogs;
with Widgets.Containers.Pulldown_Menus; use Widgets.Containers.Pulldown_Menus;
with Widgets.Menubars;                  use Widgets.Menubars;
with Widgets.Menu_Items.Menu_Checkboxes;use Widgets.Menu_Items.Menu_Checkboxes;

package body Game_Views.Ked.Menubar is

    procedure Make_Menubar( view : not null A_Ked_View;
                            win  : not null A_Window ) is
        loaded            : constant Boolean := view.Is_World_Loaded;
        mb                : A_Menubar;
        fileMenu,
        worldMenu,
        optionsMenu,
        helpMenu          : A_Pulldown_Menu;
        fileNew,
        fileOpen,
        fileImport,
        fileSave,
        fileSaveas,
        fileQuit,
        worldProperties,
        worldResize,
        optionsScaling    : A_Menu_Item;
        optionsGridsnap,
        optionsPinkScene,
        optionsOutlines   : A_Menu_Checkbox;
        helpAbout         : A_Menu_Item;
    begin
        -- File menu --
        fileNew := Create_Menu_Item( view, "miFileNew", "New..." );
        fileOpen := Create_Menu_Item( view, "miFileOpen", "Open..." );
        fileImport := Create_Menu_Item( view, "miFileImport", "Import..." );
        fileSave := Create_Menu_Item( view, "miFileSave", "Save" );
        fileSave.Set_Enabled( loaded );
        fileSaveas := Create_Menu_Item( view, "miFileSaveas", "Save As..." );
        fileSaveas.Set_Enabled( loaded );
        fileQuit := Create_Menu_Item( view, "miFileQuit", "Quit" );

        fileNew.Add_Listener( File_New'Access );
        fileOpen.Add_Listener( File_Open'Access );
        fileImport.Add_Listener( File_Import'Access );
        fileSave.Add_Listener( File_Save'Access );
        fileSaveas.Add_Listener( File_Saveas'Access );
        fileQuit.Add_Listener( File_Quit'Access );

        fileMenu := Create_Pulldown_Menu( view, "pdmFile", "File" );
        fileMenu.Add( A_Widget(fileNew) );
        fileMenu.Add( A_Widget(fileOpen) );
        fileMenu.Add( A_Widget(fileImport) );
        fileMenu.Add( A_Widget(fileSave) );
        fileMenu.Add( A_Widget(fileSaveas) );
        fileMenu.Add( A_Widget(fileQuit) );

        -- World menu --
        worldProperties := Create_Menu_Item( view, "miWorldProp", "Properties..." );
        worldProperties.Set_Enabled( loaded );
        worldResize := Create_Menu_Item( view, "miWorldResize", "Resize..." );
        worldResize.Set_Enabled( loaded );

        worldProperties.Add_Listener( World_Properties'Access );
        worldResize.Add_Listener( World_Resize'Access );

        worldMenu := Create_Pulldown_Menu( view, "pdmWorld", "World" );
        worldMenu.Add( A_Widget(worldProperties) );
        worldMenu.Add( A_Widget(worldResize) );

        -- Options menu --
        optionsGridsnap := Create_Menu_Checkbox( view, "miOptionsGrid", "Snap Items to Grid", enabled => Get_Pref( "gridsnap" ) );
        optionsPinkScene := Create_Menu_Checkbox( view, "miOptionsPink", "Pink Scene Background", enabled => Get_Pref( "pinkscene" ) );
        optionsOutlines := Create_Menu_Checkbox( view, "miOptionsOutlines", "Layer Outlines", enabled => Get_Pref( "outlines" ) );
        optionsScaling := Create_Menu_Item( view, "miOptionsScale", "Scaling Filter" );

        optionsGridsnap.Add_Listener( Options_Gridsnap'Access );
        optionsPinkScene.Add_Listener( Options_PinkScene'Access );
        optionsOutlines.Add_Listener( Options_Outlines'Access );
        optionsScaling.Add_Listener( Options_Scaling'Access );

        optionsMenu := Create_Pulldown_Menu( view, "pdmOptions", "Options" );
        optionsMenu.Add( A_Widget(optionsGridsnap) );
        optionsMenu.Add( A_Widget(optionsPinkScene) );
        optionsMenu.Add( A_Widget(optionsOutlines) );
        optionsMenu.Add( A_Widget(optionsScaling) );

        -- Help menu --
        helpAbout := Create_Menu_Item( view, "miHelpAbout", "About..." );
        helpAbout.Add_Listener( Help_About'Access );

        helpMenu := Create_Pulldown_Menu( view, "pdmHelp", "Help" );
        helpMenu.Add( A_Widget(helpAbout) );

        -- add all pulldown menus to menubar
        mb := Create_Menubar( view, "menubar" );
        mb.Add( A_Widget(fileMenu) );
        mb.Add( A_Widget(worldMenu) );
        mb.Add( A_Widget(optionsMenu) );
        mb.Add( A_Widget(helpMenu) );
        win.Set_Menubar( mb );

        -- shortcuts for menu items
        win.Add_Listener( KEY_F, (CTRL=>Yes, others=>No), Shortcut_Toggle_Filter'Access );
        win.Add_Listener( KEY_N, (CTRL=>Yes, others=>No), Shortcut_File_New'Access );
        win.Add_Listener( KEY_O, (CTRL=>Yes, others=>No), Shortcut_File_Open'Access );
        win.Add_Listener( KEY_I, (CTRL=>Yes, others=>No), Shortcut_File_Import'Access );
        win.Add_Listener( KEY_S, (CTRL=>Yes, others=>No), Shortcut_File_Save'Access );
    exception
        when others =>
            Delete( A_Widget(mb) );
            Delete( A_Widget(fileMenu) );
            Delete( A_Widget(worldMenu) );
            Delete( A_Widget(optionsMenu) );
            Delete( A_Widget(helpMenu) );
            Delete( A_Widget(fileNew) );
            Delete( A_Widget(fileOpen) );
            Delete( A_Widget(fileImport) );
            Delete( A_Widget(fileSave) );
            Delete( A_Widget(fileSaveas) );
            Delete( A_Widget(fileQuit) );
            Delete( A_Widget(worldProperties) );
            Delete( A_Widget(worldResize) );
            Delete( A_Widget(optionsScaling) );
            Delete( A_Widget(optionsGridsnap) );
            Delete( A_Widget(optionsPinkScene) );
            Delete( A_Widget(optionsOutlines) );
            Delete( A_Widget(helpAbout) );
            raise;
    end Make_Menubar;

    ----------------------------------------------------------------------------

    procedure Handle_New_Map( view : not null A_Ked_View ) is
        loaded : constant Boolean := view.Is_World_Loaded;
    begin
        view.Get_Widget( "miFileSave" ).Set_Enabled( loaded );
        view.Get_Widget( "miFileSaveas" ).Set_Enabled( loaded );
        view.Get_Widget( "miWorldProp" ).Set_Enabled( loaded );
        view.Get_Widget( "miWorldResize" ).Set_Enabled( loaded );
    end Handle_New_Map;

    ----------------------------------------------------------------------------

    procedure File_New( action : A_Menu_Action ) is
    begin
        Game_Views.Ked.Actions.Action_New( A_Ked_View(action.Get_Source.Get_View) );
    end File_New;

    ----------------------------------------------------------------------------

    procedure File_Open( action : A_Menu_Action ) is
    begin
        Game_Views.Ked.Actions.Action_Open( A_Ked_View(action.Get_Source.Get_View) );
    end File_Open;

    ----------------------------------------------------------------------------

    procedure File_Save( action : A_Menu_Action ) is
    begin
        Game_Views.Ked.Actions.Action_Save( A_Ked_View(action.Get_Source.Get_View) );
    end File_Save;

    ----------------------------------------------------------------------------

    procedure File_Saveas( action : A_Menu_Action ) is
    begin
        Game_Views.Ked.Actions.Action_Saveas( A_Ked_View(action.Get_Source.Get_View) );
    end File_Saveas;

    ----------------------------------------------------------------------------

    procedure File_Import( action : A_Menu_Action ) is
    begin
        A_Ked_View(action.Get_Source.Get_View).Do_Dialog_Action( DA_Import, saveChanges => True );
    end File_Import;

    ----------------------------------------------------------------------------

    procedure File_Quit( action : A_Menu_Action ) is
    begin
        A_Ked_View(action.Get_Source.Get_View).Do_Dialog_Action( DA_Quit, saveChanges => True );
    end File_Quit;

    ----------------------------------------------------------------------------

    procedure World_Properties( action : A_Menu_Action ) is
        view : constant A_Ked_View := A_Ked_View(action.Get_Source.Get_View);
    begin
        if view.Is_World_Loaded then
            A_Dialog(view.Get_Widget( "pnlProp" )).Show;
        end if;
    end World_Properties;

    ----------------------------------------------------------------------------

    procedure World_Resize( action : A_Menu_Action ) is
        view : constant A_Ked_View := A_Ked_View(action.Get_Source.Get_View);
    begin
        if view.Is_World_Loaded then
            A_Dialog(view.Get_Widget( "pnlResize" )).Show;
        end if;
    end World_Resize;

    ----------------------------------------------------------------------------

    procedure Options_Gridsnap( action : A_Menu_Action ) is
    begin
        if action.Get_Id = Menu_Selected then
            Set_Pref( "gridsnap", True );
        elsif action.Get_Id = Menu_Unselected then
            Set_Pref( "gridsnap", False );
        end if;
    end Options_Gridsnap;

    ----------------------------------------------------------------------------

    procedure Options_PinkScene( action : A_Menu_Action ) is
    begin
        if action.Get_Id = Menu_Selected then
            Set_Pref( "pinkscene", True );
        elsif action.Get_Id = Menu_Unselected then
            Set_Pref( "pinkscene", False );
        end if;
        action.Get_Source.Get_View.Get_Widget( "scene" ).Redraw;
    end Options_PinkScene;

    ----------------------------------------------------------------------------

    procedure Options_Outlines( action : A_Menu_Action ) is
    begin
        if action.Get_Id = Menu_Selected then
            Set_Pref( "outlines", True );
        elsif action.Get_Id = Menu_Unselected then
            Set_Pref( "outlines", False );
        end if;
        action.Get_Source.Get_View.Get_Widget( "scene" ).Redraw;
    end Options_Outlines;

    ----------------------------------------------------------------------------

    procedure Options_Scaling( action : A_Menu_Action ) is
    begin
         Game_Views.Ked.Actions.Action_Toggle_Filter( A_Ked_View(action.Get_Source.Get_View) );
    end Options_Scaling;

    ----------------------------------------------------------------------------

    procedure Help_About( action : A_Menu_Action ) is
    begin
        A_Dialog(action.Get_Source.Get_View.Get_Widget( "pnlAbout" )).Show;
    end Help_About;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Import( action : A_Key_Action; handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            Game_Views.Ked.Actions.Action_Import( A_Ked_View(action.Get_Source.Get_View) );
            handled := True;
        end if;
    end Shortcut_File_Import;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_New( action : A_Key_Action; handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            Game_Views.Ked.Actions.Action_New( A_Ked_View(action.Get_Source.Get_View) );
            handled := True;
        end if;
    end Shortcut_File_New;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Open( action : A_Key_Action; handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            Game_Views.Ked.Actions.Action_Open( A_Ked_View(action.Get_Source.Get_View) );
            handled := True;
        end if;
    end Shortcut_File_Open;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Quit( action : A_Key_Action; handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            Game_Views.Ked.Actions.Action_Quit( A_Ked_View(action.Get_Source.Get_View) );
            handled := True;
        end if;
    end Shortcut_File_Quit;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Save( action : A_Key_Action; handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            Game_Views.Ked.Actions.Action_Save( A_Ked_View(action.Get_Source.Get_View) );
            handled := True;
        end if;
    end Shortcut_File_Save;

    ----------------------------------------------------------------------------

    procedure Shortcut_Toggle_Filter( action : A_Key_Action; handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            Game_Views.Ked.Actions.Action_Toggle_Filter( A_Ked_View(action.Get_Source.Get_View) );
            handled := True;
        end if;
    end Shortcut_Toggle_Filter;

end Game_Views.Ked.Menubar;
