
with Actions;                           use Actions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Allegro.Truecolor;                 use Allegro.Truecolor;
with Applications;                      use Applications;
with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Game_Views.Ked.Menubar;
with Game_Views.Ked.Toolbar.Layers_Menu;
--with Game_Views.Ked.Toolbar.Test_Panel;
with Keyboard;                          use Keyboard;
with Support;                           use Support;
with Themes;                            use Themes;
with Themes.Palette;                    use Themes.Palette;
with Tools.Enemy_Spawners;              use Tools.Enemy_Spawners;
with Tools.Item_Spawners;               use Tools.Item_Spawners;
with Tools.Matrixbrushes;               use Tools.Matrixbrushes;
with Tools.Paintbrushes;                use Tools.Paintbrushes;
with Tools.Pointers;                    use Tools.Pointers;
with Tools.Trigger_Spawners;            use Tools.Trigger_Spawners;
with Widgets.Buttons;                   use Widgets.Buttons;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Buttons.Toggles;           use Widgets.Buttons.Toggles;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Containers.Scroll_Panes;   use Widgets.Containers.Scroll_Panes;
with Widgets.Labels;                    use Widgets.Labels;
with Widgets.Palettes;                  use Widgets.Palettes;
with Widgets.Palettes.Enemies;          use Widgets.Palettes.Enemies;
with Widgets.Palettes.Items;            use Widgets.Palettes.Items;
with Widgets.Palettes.Matrices;         use Widgets.Palettes.Matrices;
with Widgets.Palettes.Tiles;            use Widgets.Palettes.Tiles;

package body Game_Views.Ked.Toolbar is

    PALETTE_WIDTH : constant := 325;

    procedure Select_Tool( action : A_Key_Action; handled : out Boolean ) is
    begin
        if action.Get_Id = Key_Press then
            handled := True;
            case action.Get_Key is
                when KEY_P => A_Button(action.Get_Source.Get_View.Get_Widget( "btnPaint" )).Toggle_State;
                when KEY_M => A_Button(action.Get_Source.Get_View.Get_Widget( "btnMatrix" )).Toggle_State;
                when KEY_I => A_Button(action.Get_Source.Get_View.Get_Widget( "btnItem" )).Toggle_State;
                when KEY_E => A_Button(action.Get_Source.Get_View.Get_Widget( "btnEnemy" )).Toggle_State;
                when KEY_T => A_Button(action.Get_Source.Get_View.Get_Widget( "btnTrigger" )).Toggle_State;
                when KEY_S => A_Button(action.Get_Source.Get_View.Get_Widget( "btnPointer" )).Toggle_State;
                when others =>
                    handled := False;
            end case;
        else
            handled := False;
        end if;
    end Select_Tool;

    ----------------------------------------------------------------------------

    procedure Make_Toolbar( view : not null A_Ked_View; win : not null A_Window ) is
        top            : constant := Game_Views.Ked.Menubar.MENUBAR_HEIGHT;
        palTitleHeight : constant := 20;
        btnHeight      : constant := TOOLBAR_WIDTH - 2;
        scenePosWidth  : constant := 80;

        label         : A_Label;
        button        : A_Button;
        tilePalette   : A_Palette;
        matrixPalette : A_Palette;
        itemPalette   : A_Palette;
        enemyPalette  : A_Palette;
        panel         : A_Panel;
        panel2        : A_Panel;
        scrollPane    : A_Scroll_Pane;
    begin
        view.paletteGrp := Create_Button_Group;
        view.paletteGrp.Set_Keep_Selected( False );
        view.paletteGrp.Add_Listener( Action_Tool_Group'Access );

        panel := Create_Panel( view, "pnlToolbar" );
        panel.Set_Layout_LTRB( -(TOOLBAR_WIDTH-1), top, 0, TOOLBAR_WIDTH );

            button := Create_Toggle_Button( view, "btnPaint", icon => "tile_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*0, 0, -(btnHeight*1-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Show_Tiles'Access );
            view.paletteGrp.Add( button );
            panel.Add( A_Widget(button) );

            button := Create_Toggle_Button( view, "btnMatrix", icon => "matrix_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*1, 0, -(btnHeight*2-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Show_Matrices'Access );
            view.paletteGrp.Add( button );
            panel.Add( A_Widget(button) );

            button := Create_Toggle_Button( view, "btnItem", icon => "item_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*2, 0, -(btnHeight*3-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Show_Items'Access );
            view.paletteGrp.Add( button );
            panel.Add( A_Widget(button) );

            button := Create_Toggle_Button( view, "btnEnemy", icon => "enemy_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*3, 0, -(btnHeight*4-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Show_Enemies'Access );
            view.paletteGrp.Add( button );
            panel.Add( A_Widget(button) );

            panel2 := Create_Panel( view, "pnlToolSep" );
            panel2.Set_Layout_CenterHY( TOOLBAR_WIDTH-4,
                                        btnHeight*4+btnHeight/2-1,
                                        btnHeight*4+btnHeight/2 );
            panel.Add( A_Widget(panel2) );

            button := Create_Push_Button( view, "btnTrigger", icon => "trigger_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*5, 0, -(btnHeight*6-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Use_Trigger_Tool'Access );
            panel.Add( A_Widget(button) );

            button := Create_Push_Button( view, "btnSwitch", icon => "switch_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*6, 0, -(btnHeight*7-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Use_Switch_Tool'Access );
            panel.Add( A_Widget(button) );

            button := Create_Push_Button( view, "btnLift", icon=>"lift_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*7, 0, -(btnHeight*8-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            panel.Add( A_Widget(button) );

            button := Create_Push_Button( view, "btnPortal", icon => "portal_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*8, 0, -(btnHeight*9-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            panel.Add( A_Widget(button) );

            button := Create_Push_Button( view, "btnPointer", icon => "pointer_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*9, 0, -(btnHeight*10-1) );
            button.Set_Border( Border_None );
            button.Set_Color( Foreground, Get_Application.Get_Theme.Color( Selected ) );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Action_Use_Pointer'Access );
            panel.Add( A_Widget(button) );

        win.Add( A_Widget(panel) );

        panel := Create_Panel( view, "pnlScenePos" );
        panel.Set_Layout_LTRB( -(TOOLBAR_WIDTH+scenePosWidth), -(btnHeight+1), TOOLBAR_WIDTH, 0 );

            label := Create_Label( view, "lblScenePos", "" );
            label.Set_Align( Align_Right );
            label.Set_Layout_LTRB( -75, 0, 5, 0 );
            panel.Add( A_Widget(label) );

        win.Add( A_Widget(panel) );

        panel := Create_Panel( view, "pnlPalette" );
        panel.Set_Layout_LTRB( -(PALETTE_WIDTH+TOOLBAR_WIDTH-1), top, TOOLBAR_WIDTH, 0 );
        panel.Set_Visible( False );

            panel2 := Create_Panel( view, "pnlPaletteTop" );
            panel2.Set_Layout_LTRB( 0, 0, 0, -(palTitleHeight-1) );
            panel2.Set_Color( Foreground, panel2.Get_Color( Selected ) );
            panel2.Set_Border( Border_None );

                label := Create_Label( view, "lblPalTitle", "" );
                label.Set_Layout_LTRB( 4, 0, palTitleHeight + 4, 0 );
                label.Set_Color( Text, base_colors(White) );
                label.Set_Color( Foreground, label.Get_Color( Selected ) );
                panel2.Add( A_Widget(label) );

                button := Create_Toggle_Button( view, "btnTack", icon => "thumbtack_icon" );
                button.Set_Layout_LTRB( -(palTitleHeight-1) + 2, 2, 2, 2 );
                button.Set_Focusable( False );
                button.Set_Color( Foreground, button.Get_Color( Selected ) );
                button.Add_Listener( Action_Pin_Palette'Access );
                panel2.Add( A_Widget(button) );

            panel.Add( A_Widget(panel2) );

            panel2 := Create_Panel( view, "pnlPaletteBottom" );
            panel2.Set_Layout_LTRB( 0, palTitleHeight + 1, 0, 0 );
            panel2.Set_Border( Border_None );

                scrollPane := Create_Scroll_Pane( view, "spnTile" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                tilePalette := Create_Tile_Palette( view, "tilePalette" );
                    tilePalette.Set_Icon_Size( 16, 16 );
                    tilePalette.Set_Columns( 18 );
                    scrollPane.Set_Client( A_Widget(tilePalette) );
                panel2.Add( A_Widget(scrollPane) );

                scrollPane := Create_Scroll_Pane( view, "spnMatrix" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    matrixPalette := Create_Matrix_Palette( view, "matrixPalette" );
                    matrixPalette.Set_Icon_Size( 16, 16 );
                    matrixPalette.Set_Columns( 18 );
                    scrollPane.Set_Client( A_Widget(matrixPalette) );
                panel2.Add( A_Widget(scrollPane) );

                scrollPane := Create_Scroll_Pane( view, "spnItem" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Draw_Vbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    itemPalette := Create_Item_Palette( view, "itemPalette" );
                    itemPalette.Set_Icon_Size( 32, 32 );
                    itemPalette.Set_Columns( Floor( Float(PALETTE_WIDTH) / 33.0 ) );
                    scrollPane.Set_Client( A_Widget(itemPalette) );
                panel2.Add( A_Widget(scrollPane) );

                scrollPane := Create_Scroll_Pane( view, "spnEnemy" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Draw_Vbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    enemyPalette := Create_Enemy_Palette( view, "enemyPalette" );
                    enemyPalette.Set_Icon_Size( 48, 48 );
                    enemyPalette.Set_Columns( Floor( Float(PALETTE_WIDTH) / 49.0 ) );
                    scrollPane.Set_Client( A_Widget(enemyPalette) );
                panel2.Add( A_Widget(scrollPane) );

            panel.Add( A_Widget(panel2) );

        win.Add( A_Widget(panel) );

--        Test_Panel.Make_Test_Panel( view, win );
        Layers_Menu.Make_Menu( view, win );

        win.Add_Listener( KEY_S, (others=>No), Select_Tool'Access );  -- select/pointer tool
        win.Add_Listener( KEY_P, (others=>No), Select_Tool'Access );  -- tile palette
        win.Add_Listener( KEY_M, (others=>No), Select_Tool'Access );  -- matrix palette
        win.Add_Listener( KEY_I, (others=>No), Select_Tool'Access );  -- item palette
        win.Add_Listener( KEY_E, (others=>No), Select_Tool'Access );  -- enemy palette
        win.Add_Listener( KEY_T, (others=>No), Select_Tool'Access );  -- trigger tool
    exception
        when others =>
            Delete( A_Widget(label) );
            Delete( A_Widget(button) );
            Delete( A_Widget(tilePalette) );
            Delete( A_Widget(matrixPalette) );
            Delete( A_Widget(itemPalette) );
            Delete( A_Widget(enemyPalette) );
            Delete( A_Widget(panel) );
            Delete( A_Widget(panel2) );
            Delete( A_Widget(scrollPane) );
            raise;
    end Make_Toolbar;

    ----------------------------------------------------------------------------

    procedure Handle_New_Map( view : not null A_Ked_View ) is
    begin
        --view.Get_Widget( "btnTack" ).Set_Enabled( True );
        view.Get_Widget( "btnPaint" ).Set_Enabled( True );
        view.Get_Widget( "btnMatrix" ).Set_Enabled( True );
        view.Get_Widget( "btnItem" ).Set_Enabled( True );
        view.Get_Widget( "btnEnemy" ).Set_Enabled( True );
        view.Get_Widget( "btnTrigger" ).Set_Enabled( True );
        view.Get_Widget( "btnSwitch" ).Set_Enabled( True );
        view.Get_Widget( "btnLift" ).Set_Enabled( True );
        view.Get_Widget( "btnPortal" ).Set_Enabled( True );
        view.Get_Widget( "btnPointer" ).Set_Enabled( True );
    end Handle_New_Map;

    ----------------------------------------------------------------------------

    procedure Action_Pin_Palette( action : A_Button_Action ) is
    begin
       if action.Get_Id = Press then
           A_Ked_View(action.Get_Source.Get_View).paletteGrp.Set_Keep_Selected( True );
       elsif action.Get_Id = Release then
           A_Ked_View(action.Get_Source.Get_View).paletteGrp.Set_Keep_Selected( False );
       end if;
    end Action_Pin_Palette;

    ----------------------------------------------------------------------------

    procedure Action_Show_Enemies( action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "spnEnemy" ).Set_Visible( True );
            action.Get_Source.Get_View.Get_Widget( "btnEnemy" ).Set_Border( Border_Raised );
            A_Label(action.Get_Source.Get_View.Get_Widget( "lblPalTitle" )).Set_Text( "Enemies" );
        elsif action.Get_Id = Release then
            action.Get_Source.Get_View.Get_Widget( "spnEnemy" ).Set_Visible( False );
            action.Get_Source.Get_View.Get_Widget( "btnEnemy" ).Set_Border( Border_None );
        end if;
    end Action_Show_Enemies;

    ----------------------------------------------------------------------------

    procedure Action_Show_Items( action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "spnItem" ).Set_Visible( True );
            action.Get_Source.Get_View.Get_Widget( "btnItem" ).Set_Border( Border_Raised );
            A_Label(action.Get_Source.Get_View.Get_Widget( "lblPalTitle" )).Set_Text( "Items" );
        else
            action.Get_Source.Get_View.Get_Widget( "spnItem" ).Set_Visible( False );
            action.Get_Source.Get_View.Get_Widget( "btnItem" ).Set_Border( Border_None );
        end if;
    end Action_Show_Items;

    ----------------------------------------------------------------------------

    procedure Action_Show_Matrices( action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "spnMatrix" ).Set_Visible( True );
            action.Get_Source.Get_View.Get_Widget( "btnMatrix" ).Set_Border( Border_Raised );
            A_Label(action.Get_Source.Get_View.Get_Widget( "lblPalTitle" )).Set_Text( "Tile Matrices" );
        elsif action.Get_Id = Release then
            action.Get_Source.Get_View.Get_Widget( "spnMatrix" ).Set_Visible( False );
            action.Get_Source.Get_View.Get_Widget( "btnMatrix" ).Set_Border( Border_None );
        end if;
    end Action_Show_Matrices;

    ----------------------------------------------------------------------------

    procedure Action_Show_Tiles( action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "spnTile" ).Set_Visible( True );
            action.Get_Source.Get_View.Get_Widget( "btnPaint" ).Set_Border( Border_Raised );
            A_Label(action.Get_Source.Get_View.Get_Widget( "lblPalTitle" )).Set_Text( "Tiles" );
        elsif action.Get_Id = Release then
            action.Get_Source.Get_View.Get_Widget( "spnTile" ).Set_Visible( False );
            action.Get_Source.Get_View.Get_Widget( "btnPaint" ).Set_Border( Border_None );
        end if;
    end Action_Show_Tiles;

    ----------------------------------------------------------------------------

    procedure Action_Tool_Group( action : A_Button_Group_Action ) is
    begin
        if action.Get_Id = Group_Set then
            action.Get_Source.Get_View.Get_Widget( "pnlPalette" ).Set_Visible( True );
            action.Get_Source.Get_View.Get_Widget( "scene" ).Set_Layout_LTRB( 0, Game_Views.Ked.Menubar.MENUBAR_HEIGHT, TOOLBAR_WIDTH + PALETTE_WIDTH, 0 );
        elsif action.Get_Id = Group_Cleared then
            action.Get_Source.Get_View.Get_Widget( "pnlPalette" ).Set_Visible( False );
            action.Get_Source.Get_View.Get_Widget( "scene" ).Set_Layout_LTRB( 0, Game_Views.Ked.Menubar.MENUBAR_HEIGHT, TOOLBAR_WIDTH, 0 );
        end if;
    end Action_Tool_Group;

    ----------------------------------------------------------------------------

    procedure Action_Use_Pointer( action : A_Button_Action ) is
        tool : A_Tool;
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "btnPointer" ).Set_Border( Border_Raised );
        elsif action.Get_Id = Release then
            action.Get_Source.Get_View.Get_Widget( "btnPointer" ).Set_Border( Border_None );
            tool := Create_Pointer;
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
    end Action_Use_Pointer;

    ----------------------------------------------------------------------------

    procedure Action_Use_Switch_Tool( action : A_Button_Action ) is
        --tool : A_Tool;
    begin
        if action.Get_Id = Release then
            null;
            pragma Debug( Dbg( "STUB: Action_Use_Switch_Tool" ) );
            --tool := Create_Switch_Tool;
            --Queue_Set_Tool( tool );
            --Delete( tool );
        end if;
    end Action_Use_Switch_Tool;

    ----------------------------------------------------------------------------

    procedure Action_Use_Trigger_Tool( action : A_Button_Action ) is
        tool : A_Tool;
    begin
        if action.Get_Id = Press then
            action.Get_Source.Get_View.Get_Widget( "btnTrigger" ).Set_Border( Border_Raised );
        elsif action.Get_Id = Release then
            action.Get_Source.Get_View.Get_Widget( "btnTrigger" ).Set_Border( Border_None );
            tool := Create_Trigger_Spawner;
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
    end Action_Use_Trigger_Tool;

    ----------------------------------------------------------------------------

    procedure Handle_Tool_Changed( view : not null A_Ked_View;
                                   tool : not null A_Tool ) is
        theme : constant A_Theme := Get_Application.Get_Theme;

        function Button_Color( on : Boolean ) return Color_Type is
        begin
            if on then
                return theme.Color( Selected );
            end if;
            return theme.Color( Foreground );
        end Button_Color;

    begin
        if not A_Button(view.Get_Widget( "btnTack" )).Get_State then
            view.paletteGrp.Unset;
        end if;
        view.Get_Widget( "btnPaint"   ).Set_Color( Foreground, Button_Color( tool.all in Paintbrush'Class ) );
        view.Get_Widget( "btnMatrix"  ).Set_Color( Foreground, Button_Color( tool.all in Matrixbrush'Class ) );
        view.Get_Widget( "btnItem"    ).Set_Color( Foreground, Button_Color( tool.all in Item_Spawner'Class ) );
        view.Get_Widget( "btnEnemy"   ).Set_Color( Foreground, Button_Color( tool.all in Enemy_Spawner'Class ) );
        view.Get_Widget( "btnTrigger" ).Set_Color( Foreground, Button_Color( tool.all in Trigger_Spawner'Class ) );
      --view.Get_Widget( "btnSwitch"  ).Set_Color( Foreground, Button_Color( tool.all in Switch_Tool'Class ) );
        view.Get_Widget( "btnPointer" ).Set_Color( Foreground, Button_Color( tool.all in Pointer'Class ) );
    end Handle_Tool_Changed;

end Game_Views.Ked.Toolbar;
