
with Actions;                           use Actions;
with Ada.Exceptions;                    use Ada.Exceptions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Applications;                      use Applications;
with Debugging;                         use Debugging;
with Directions;                        use Directions;
with Engine_Debugging;                  use Engine_Debugging;
with Events.Application;                use Events.Application;
with Events.Game;                       use Events.Game;
with Events.Ked;                        use Events.Ked;
with Events.World;                      use Events.World;
with Game_Views;                        use Game_Views;
with Game_Views.Ked.Actions;
with Game_Views.Ked.Toolbar;
with Game_Views.Ked.Toolbar.Layers_Menu;
with Game_Views.Ked.Menubar;
with GNAT.Directory_Operations;
with Keyboard;                          use Keyboard;
with Preferences;                       use Preferences;
with Scaling;                           use Scaling;
with Support.Paths;                     use Support.Paths;
with Widgets.Containers;                use Widgets.Containers;
with Widgets.Containers.Panels.Dialogs; use Widgets.Containers.Panels.Dialogs;
with Widgets.Containers.Panels.Dialogs.About;
 use Widgets.Containers.Panels.Dialogs.About;
with Widgets.Containers.Panels.Dialogs.Import_World;
 use Widgets.Containers.Panels.Dialogs.Import_World;
with Widgets.Containers.Panels.Dialogs.New_World;
 use Widgets.Containers.Panels.Dialogs.New_World;
with Widgets.Containers.Panels.Dialogs.Open_World;
 use Widgets.Containers.Panels.Dialogs.Open_World;
with Widgets.Containers.Panels.Dialogs.Properties;
 use Widgets.Containers.Panels.Dialogs.Properties;
with Widgets.Containers.Panels.Dialogs.Resize_World;
 use Widgets.Containers.Panels.Dialogs.Resize_World;
with Widgets.Containers.Panels.Dialogs.Save_Changes;
 use Widgets.Containers.Panels.Dialogs.Save_Changes;
with Widgets.Containers.Panels.Dialogs.Save_World;
 use Widgets.Containers.Panels.Dialogs.Save_World;
with Widgets.Containers.Scenes.Ked;     use Widgets.Containers.Scenes.Ked;
with Worlds;

pragma Elaborate_All( Game_Views );
pragma Elaborate_All( Preferences );

package body Game_Views.Ked is

    procedure Scroll_Scene( action : A_Key_Action; handled : out Boolean ) is
    begin
        if action.Get_Id = Key_Held then
            handled := True;
            case action.Get_Key is
                when KEY_LEFT  => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Scroll_Increment( Left );
                when KEY_RIGHT => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Scroll_Increment( Right );
                when KEY_UP    => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Scroll_Increment( Up );
                when KEY_DOWN  => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Scroll_Increment( Down );
                when others =>
                    handled := False;
                    return;
            end case;
        else
            handled := False;
        end if;
    end Scroll_Scene;

    ----------------------------------------------------------------------------

    procedure Zoom_Scene( action : A_Key_Action; handled : out Boolean ) is
    begin
        if action.Get_Id = Key_Press then
            handled := True;
            case action.Get_Key is
                when KEY_0      => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Zoom_Identity;
                when KEY_MINUS  => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Zoom_Out;
                when KEY_EQUALS => A_Ked_Scene(action.Get_Source.Get_View.Get_Widget( "scene" )).Zoom_In;
                when others =>
                    handled := False;
                    return;
            end case;
        else
            handled := False;
        end if;
    end Zoom_Scene;

    ----------------------------------------------------------------------------

    function Create_Game_View( xres, yres : Positive;
                               scale      : Positive ) return A_Game_View is
        this : A_Ked_View := new Ked_View;
    begin
        this.Construct;
        this.Populate_View( xres, yres, scale );
        return A_Game_View(this);
    exception
        when others =>
            Delete( A_Game_View(this) );
            raise;
    end Create_Game_View;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Ked_View ) is
    begin
        Delete( this.paletteGrp );
        Delete( this.layerGrp );
        Game_View(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    procedure Do_Dialog_Action( this        : access Ked_View;
                                action      : Dialog_Action;
                                saveChanges : Boolean ) is
    begin
        if action = DA_Save then
            this.nextAction := DA_None;
            A_Dialog(this.Get_Widget( "pnlSaveWorld" )).Show;
        elsif not this.Is_Modified or else not saveChanges then
            case action is
                when DA_Import => A_Dialog(this.Get_Widget( "pnlImport" )).Show;
                when DA_New    => A_Dialog(this.Get_Widget( "pnlNewWorld" )).Show;
                when DA_Open   => A_Dialog(this.Get_Widget( "pnlOpenWorld" )).Show;
                when DA_Quit   => Queue_Close_Window;
                when DA_Save   |
                     DA_None   => null;
            end case;
        else
            this.nextAction := action;
            A_Dialog(this.Get_Widget( "pnlAskSave" )).Show;
        end if;
    end Do_Dialog_Action;

    ----------------------------------------------------------------------------

    procedure Do_Dialog_Action( this : access Ked_View; saveChanges : Boolean ) is
    begin
        if not this.Is_Modified or else not saveChanges then
            this.Do_Dialog_Action( this.nextAction, saveChanges => False );
        else
            if Length( this.filename ) > 0 then
                begin
                    this.Save_World( To_String( this.filename ) );
                    this.Do_Dialog_Action( this.nextAction, saveChanges => False );
                exception
                    when e : others =>
                        -- shouldn't happen unless the drive disconnected or the
                        -- file is read-only. don't do whatever dialog action
                        -- we were going to do, to avoid losing changes.
                        pragma Debug( Dbg( "Error saving world: " &
                                           Exception_Message( e ),
                                           D_VIEW, ERROR ) );
                        this.nextAction := DA_None;
                end;
            else
                A_Dialog(this.Get_Widget( "pnlSaveWorld" )).Show;
            end if;
        end if;
    end Do_Dialog_Action;

    ----------------------------------------------------------------------------

    function Get_Filename( this : not null access Ked_View'Class ) return String is
    begin
        return To_String( this.filename );
    end Get_Filename;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Close_Request( this : access Ked_View ) is
    begin
         Actions.Action_Quit( A_Ked_View(this) );
    end Handle_Close_Request;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Ked_View;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
        pragma Unreferenced( resp );

        ------------------------------------------------------------------------

        procedure Do_World_Property_Changed( evt : A_World_Property_Changed_Event ) is
            property : constant String := evt.Get_Property_Name;
        begin
            if property = "filename" then
                this.Set_Filename( evt.Get_Value );
            end if;
        end Do_World_Property_Changed;

        ------------------------------------------------------------------------

    begin
        Game_View(this.all).Handle_Event( evt, resp );
        if evt /= null then
            if evt.Get_Id = WORLD_MODIFIED_ID then
                this.Set_Modified( True );
            elsif evt.Get_Id = SET_TOOL_ID then
                Toolbar.Handle_Tool_Changed( A_Ked_View(this), A_Set_Tool_Event(evt).Get_Tool );
            elsif evt.Get_Id = NEW_WORLD_ID then
                this.loaded := True;
                this.Set_Modified( False );
                Menubar.Handle_New_Map( A_Ked_View(this) );
                Toolbar.Handle_New_Map( A_Ked_View(this) );
                Toolbar.Layers_Menu.Update_Menu( A_Ked_View(this), A_New_World_Event(evt).Get_Layers );
                Queue_View_Ready( True );
            elsif evt.Get_Id = WORLD_PROPERTY_CHANGED_ID then
                Do_World_Property_Changed( A_World_Property_Changed_Event(evt) );
            end if;
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Paused( this : access Ked_View; paused : Boolean ) is
        pragma Unreferenced( this );
    begin
        null;
        pragma Debug( paused, Dbg( "Game_View.Ked: Game paused", D_VIEW, INFO ) );
        pragma Debug( not paused, Dbg( "Game_View.Ked: Game resumed", D_VIEW, INFO ) );
    end Handle_Paused;

    ----------------------------------------------------------------------------

    function Is_Modified( this : not null access Ked_View'Class ) return Boolean is
    begin
        return this.modified;
    end Is_Modified;

    ----------------------------------------------------------------------------

    function Is_World_Loaded( this : not null access Ked_View'Class ) return Boolean is
    begin
        return this.loaded;
    end Is_World_Loaded;

    ----------------------------------------------------------------------------

    procedure Populate_View( this  : not null access Ked_View'Class;
                             xres,
                             yres,
                             scale : Integer ) is
        win    : A_Window;
        scene  : A_Ked_Scene;
        dialog : A_Dialog;
    begin
        win := Create_Window( this, "win",
                              xres, yres, scale,
                              To_Filter( Get_Pref( "scaling_filter" ) ) );

        -- [ scene ] --
        scene := Create_Scene( this, "scene" );
        scene.Set_Layout_LTRB( 0, Menubar.MENUBAR_HEIGHT, Toolbar.TOOLBAR_WIDTH, 0 );
        win.Add( A_Widget(scene) );

        -- add key handlers to the window to control the scene
        win.Add_Listener( KEY_LEFT,   Scroll_Scene'Access );
        win.Add_Listener( KEY_RIGHT,  Scroll_Scene'Access );
        win.Add_Listener( KEY_UP,     Scroll_Scene'Access );
        win.Add_Listener( KEY_DOWN,   Scroll_Scene'Access );
        win.Add_Listener( KEY_0,      (others=>No), Zoom_Scene'Access );
        win.Add_Listener( KEY_MINUS,  (others=>No), Zoom_Scene'Access );
        win.Add_Listener( KEY_EQUALS, (others=>No), Zoom_Scene'Access );

        -- [ tool bar ] --
        Toolbar.Make_Toolbar( A_Ked_View(this), win );

        -- [ dialogs ] --
        dialog := Create_New_Dialog( this, "pnlNewWorld" );
        win.Add( A_Widget(dialog) );
        dialog := Create_Open_Dialog( this, "pnlOpenWorld" );
        win.Add( A_Widget(dialog) );
        dialog := Create_Save_Dialog( this, "pnlSaveWorld" );
        win.Add( A_Widget(dialog) );
        dialog := Create_Save_Changes_Dialog( this, "pnlAskSave" );
        win.Add( A_Widget(dialog) );
        dialog := Create_Import_Dialog( this, "pnlImport" );
        win.Add( A_Widget(dialog) );
        dialog := Create_Properties_Dialog( this, "pnlProp" );
        win.Add( A_Widget(dialog) );
        dialog := Create_Resize_Dialog( this, "pnlResize" );
        win.Add( A_Widget(dialog) );
        dialog := Create_About_Dialog( this, "pnlAbout" );
        win.Add( A_Widget(dialog) );

        -- [ menu bar ] --
        Menubar.Make_Menubar( A_Ked_View(this), win );

        this.Set_Window( win );
    exception
        when others =>
            Delete( A_Widget(dialog) );
            Delete( A_Widget(scene) );
            Delete( A_Widget(win) );
            raise;
    end Populate_View;

    ----------------------------------------------------------------------------

    not overriding
    procedure Save_World( this : access Ked_View; filename : String ) is
        l_filename : constant String := Add_Extension( filename, Worlds.World_Extension, force => True );
    begin
        Trigger_Save_World( l_filename );   -- may raise exception
        this.Set_Filename( l_filename );
        this.Set_Modified( False );
    end Save_World;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Gridsnap( this : access Ked_View; enabled : Boolean ) is
    begin
        this.gridsnap := enabled;
    end Set_Gridsnap;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Filename( this : access Ked_View; filename : String ) is
    begin
        if filename'Length /= Length( this.filename ) or else
           filename /= To_String( this.filename )
        then
            this.filename := To_Unbounded_String( filename );
            this.Update_Win_Title;
        end if;
    end Set_Filename;

    ----------------------------------------------------------------------------

    not overriding
    procedure Set_Modified( this : access Ked_View; modified : Boolean ) is
    begin
        if this.modified /= modified then
            this.modified := modified;
            this.Update_Win_Title;
        end if;
    end Set_Modified;

    ----------------------------------------------------------------------------

    overriding
    procedure Start( this : access Ked_View ) is
    begin
        Game_View(this.all).Start;
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "New_World" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "Set_Tool" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "World_Property_Changed" );
        this.Get_Corral.Add_Listener( A_Event_Listener(this), "World_Modified" );
        Queue_New_Game;
    end Start;

    ----------------------------------------------------------------------------

    overriding
    procedure Stop( this : access Ked_View ) is
    begin
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "World_Modified" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "World_Property_Changed" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "Set_Tool" );
        this.Get_Corral.Remove_Listener( A_Event_Listener(this), "New_World" );
        Game_View(this.all).Stop;
    end Stop;

    ----------------------------------------------------------------------------

    not overriding
    procedure Toggle_Palette( this : access Ked_View; on : Boolean ) is
    begin
        this.paletteOn := on;


        Dbg( "Ked_View.paletteOn? " & on'Img );
--        this.Get_Widget( "palette" ).Set_Visible( this.paletteOn );
--        if this.paletteOn then
--        else
--        end if;
    end Toggle_Palette;

    ----------------------------------------------------------------------------

    not overriding
    procedure Update_Win_Title( this : access Ked_View ) is

        ------------------------------------------------------------------------

        function Make_Filename return String is
        begin
            if Length( this.filename ) > 0 then
                return GNAT.Directory_Operations.File_Name( To_String( this.filename ) );
            else
                return "Untitled";
            end if;
        end Make_Filename;

        ------------------------------------------------------------------------

        function Make_Modified return String is
        begin
            if this.modified then
                return " (modified)";
            else
                return "";
            end if;
        end Make_Modified;

        ------------------------------------------------------------------------

    begin
        this.win.Set_Title( Get_Application.Get_Window_Title & " - " &
                            Make_Filename & Make_Modified );
    end Update_Win_Title;

begin

    Preferences.Set_Default( "gridsize", 16 );
    Preferences.Set_Default( "gridsnap", True );
    Preferences.Set_Default( "pinkscene", False );
    Preferences.Set_Default( "outlines", True );
    Preferences.Set_Default( "draw_clipping", False );
    Preferences.Set_Default( "scaling_filter", "Quality" );

    Register_Allocator( Create_Game_View'Access );

end Game_Views.Ked;
