
--with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Item_Spawners is

    function Create_Item_Spawner( id : String ) return A_Tool is
        this : constant A_Item_Spawner := new Item_Spawner;
    begin
        this.Construct( id );
        return A_Tool(this);
    end Create_Item_Spawner;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this      : access Item_Spawner;
                     func      : Function_Type;
                     modifiers : Modifiers_Array;
                     first     : Boolean;
                     world     : not null A_World;
                     worldX,
                     worldY,
                     layer     : Integer ) is
        pragma Unreferenced( modifiers, layer );

        ------------------------------------------------------------------------

        function Snap( n : Integer ) return Float is
        begin
            if Get_Pref( "gridsnap" ) then
                return Float(Support.Grid_Snap( n, Get_Pref( "gridsize" ), centered => True ));
            end if;
            return Float(n);
        end Snap;

        ------------------------------------------------------------------------

        tool : A_Tool;
    begin
        if first then
            if func = Primary then
                if worldX >= 0 and then worldX < world.Get_Width and then
                   worldY >= 0 and then worldY < world.Get_Height
                then
                    world.Spawn_Entity( To_String( this.id ),
                                        Snap( worldX ), Snap( worldY ) );
                end if;
            elsif func = Secondary or else func = Tertiary then
                tool := Create_Pointer;
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Item_Spawner; id : String ) is
    begin
        Tool(this.all).Construct;
        this.id := To_Unbounded_String( id );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_ID( this : not null access Item_Spawner'Class ) return String is
    begin
        return To_String( this.id );
    end Get_ID;

end Tools.Item_Spawners;
