
--with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Tools.Paintbrushes;                use Tools.Paintbrushes;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Matrixbrushes is

    function Create_Matrixbrush( matrix  : not null A_Tile_Matrix;
                                 offsetX,
                                 offsetY : Integer ) return A_Tool is
        this : constant A_Matrixbrush := new Matrixbrush;
    begin
        this.Construct( matrix, offsetX, offsetY );
        return A_Tool(this);
    end Create_Matrixbrush;

    ----------------------------------------------------------------------------

    overriding
    procedure Adjust( this : access Matrixbrush ) is
    begin
        Tool(this.all).Adjust;
        this.matrix := Copy( this.matrix );
    end Adjust;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this      : access Matrixbrush;
                     func      : Function_Type;
                     modifiers : Modifiers_Array;
                     first     : Boolean;
                     world     : not null A_World;
                     worldX,
                     worldY,
                     layer     : Integer ) is
        tileX  : constant Integer := worldX / world.Tile_Width;
        tileY  : constant Integer := worldY / world.Tile_Width;
        offX,
        offY   : Integer;
        width,
        height : Integer;
        tool   : A_Tool;
    begin
        if func = Primary then
            if first then
                this.startTileX := tileX;
                this.startTileY := tileY;
                this.overTileX := -1;
                this.overTileY := -1;
            end if;
            if tileX /= this.overTileX or else tileY /= this.overTileY then
                this.overTileX := tileX;
                this.overTileY := tileY;
                height := this.matrix'Length(1);
                width := this.matrix'Length(2);
                if modifiers(CTRL) then
                    -- Auto-Offset constraint: draw the matrix as though the
                    -- first draw location is at offset of the matrix.
                    offX := width - (((this.startTileX - 1 - this.offsetX) mod width) + 1);
                    offY := height - (((this.startTileY - 1 - this.offsetY) mod height) + 1);
                else
                    offX := this.offsetX;
                    offY := this.offsetY;
                end if;
                world.Set_Tile( layer, tileX, tileY,
                                this.matrix(
                                    ((tileY + offY) mod height) + 1,
                                    ((tileX + offX) mod width) + 1
                                )
                              );
            end if;
        elsif first then
            if func = Secondary then
                if tileX < world.Get_Width_Tiles and then
                   tileY < world.Get_Height_Tiles
                then
                    tool := Create_Paintbrush( world.Get_Tile_Id( layer, tileX, tileY ) );
                    Queue_Set_Tool( tool );
                    Delete( tool );
                end if;
            elsif func = Tertiary then
                tool := Create_Pointer;
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this     : access Matrixbrush;
                         matrix   : A_Tile_Matrix;
                         offsetX,
                         offsetY  : Integer ) is
    begin
        Tool(this.all).Construct;
        this.matrix := Copy( matrix );
        this.offsetX := offsetX;
        this.offsetY := offsetY;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Matrixbrush ) is
    begin
        Delete( this.matrix );
        Tool(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Matrix( this : not null access Matrixbrush'Class ) return A_Tile_Matrix is
    begin
        return this.matrix;
    end Get_Matrix;

    ----------------------------------------------------------------------------

    procedure Get_Offset( this : not null access Matrixbrush'Class;
                          x, y : out Integer ) is
    begin
        x := this.offsetX;
        y := this.offsetY;
    end Get_Offset;

end Tools.Matrixbrushes;
