
with Ada.Exceptions;                    use Ada.Exceptions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Events.Ked;                        use Events.Ked;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Support.Paths;                     use Support.Paths;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Input_Boxes;               use Widgets.Input_Boxes;
with Widgets.Input_Boxes.Constraints;   use Widgets.Input_Boxes.Constraints;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.Import_World is

    function Create_Import_Dialog( view : not null access Game_Views.Game_View'Class;
                                   id   : String ) return A_Dialog is
        this : A_Dialog := new Import_Dialog;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Import_Dialog;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Import_Dialog;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        label     : A_Label;
        input_box : A_Input_Box;
        button    : A_Button;
    begin
        Dialog(this.all).Construct( view, id, "Import World...", "" );
        this.Set_Layout_Center( 200, 162 );

        label := Create_Label( view, this.id & ":lblImage", "Image filename [*.bmp]:" );
        label.Set_Layout_LTRB( 5, 5, 5, -23 );
        this.Add( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibImage" );
        input_box.Set_Layout_LTRB( 5, 26, 5, -44 );
        input_box.Set_Constraint( Constraints.Filename_Only'Access );
        input_box.Set_Max_Length( 255 );
        input_box.Set_Prev( this.id & ":btnCancel" );
        input_box.Set_Next( this.id & ":ibLib" );
        this.Add( A_Widget(input_box) );

        label := Create_Label( view, this.id & ":lblLib", "Library:" );
        label.Set_Layout_LTRB( 5, 47, -60, -65 );
        this.Add( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibLib" );
        input_box.Set_Layout_LTRB( 65, 47, 5, -65 );
        input_box.Set_Constraint( Constraints.Basename_Only'Access );
        input_box.Set_Prev( this.id & ":ibImage" );
        input_box.Set_Next( this.id & ":ibDomain" );
        this.Add( A_Widget(input_box) );

        label := Create_Label( view, this.id & ":lblDomain", "Domain:" );
        label.Set_Layout_LTRB( 5, 68, -60, -86 );
        this.Add( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibDomain" );
        input_box.Set_Layout_LTRB( 65, 68, 5, -86 );
        input_box.Set_Constraint( Constraints.Basename_Only'Access );
        input_box.Set_Prev( this.id & ":ibLib" );
        input_box.Set_Next( this.id & ":ibTolerance" );
        this.Add( A_Widget(input_box) );

        label := Create_Label( view, this.id & ":lblTolerance", "Tolerance:" );
        label.Set_Layout_LTRB( 5, 89, -60, -107 );
        this.Add( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibTolerance" );
        input_box.Set_Layout_LTRB( 65, 89, 5, -107 );
        input_box.Set_Constraint( Constraints.Int_Only'Access );
        input_box.Set_Max_Length( 3 );
        input_box.Set_Text( "0" );
        input_box.Set_Prev( this.id & ":ibDomain" );
        input_box.Set_Next( this.id & ":btnOk" );
        this.Add( A_Widget(input_box) );

        button := Create_Push_Button( view, this.id & ":btnOk", "Import" );
        button.Set_Layout_LTWH( -63, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":ibTolerance" );
        button.Set_Next( this.id & ":btnCancel" );
        this.Add( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( -5, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnOk" );
        button.Set_Next( this.id & ":ibImage" );
        this.Add( A_Widget(button) );
    exception
        when others =>
            Delete( A_Widget(label) );
            Delete( A_Widget(input_box) );
            Delete( A_Widget(button) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Import_Dialog;
                             action : A_Button_Action ) is

        procedure Ok_Button is
            view      : constant A_Game_View := A_Game_View(this.Get_View);
            image     : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibImage" )).Get_Text;
            libName   : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibLib" )).Get_Text;
            domain    : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibDomain" )).Get_Text;
            tolerance : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibTolerance" )).Get_Text;
        begin
            if image'Length     > 0 and then
               libName'Length   > 0 and then
               domain'Length    > 0 and then
               tolerance'Length > 0
            then
                Trigger_Import_World( Add_Extension( image, "bmp" ),
                                      libName, domain, Integer'Value( tolerance ) );
                this.Hide;
            end if;
        exception
            when e : others =>
                null;
                pragma Debug( Dbg( "Error importing world: " &
                                   Exception_Message( e ), D_VIEW, ERROR ) );
        end Ok_Button;

    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnOk" ) then
                Ok_Button;
            elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
                this.Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Import_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = KEY_ESC and then action.Get_Id = Key_Press then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Show( this : access Import_Dialog ) is
        view : constant A_Game_View := A_Game_View(this.Get_View);
    begin
        A_Input_Box(view.Get_Widget( this.id & ":ibImage" )).Set_Text( "" );
        A_Input_Box(view.Get_Widget( this.id & ":ibLib" )).Set_Text( "" );
        A_Input_Box(view.Get_Widget( this.id & ":ibDomain" )).Set_Text( "" );
        A_Input_Box(view.Get_Widget( this.id & ":ibTolerance" )).Set_Text( "0" );
    end Handle_Show;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access Import_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_Window.Give_Focus( this.Get_View.Get_Widget( this.id & ":ibImage" ) );
    end Show;

end Widgets.Containers.Panels.Dialogs.Import_World;
