
with Ada.Exceptions;                    use Ada.Exceptions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Events.World;                      use Events.World;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Input_Boxes;               use Widgets.Input_Boxes;
with Widgets.Input_Boxes.Constraints;   use Widgets.Input_Boxes.Constraints;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.Resize_World is

    function Create_Resize_Dialog( view : not null access Game_Views.Game_View'Class;
                                   id   : String ) return A_Dialog is
        this : A_Dialog := new Resize_Dialog;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Resize_Dialog;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Resize_Dialog;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        button   : A_Button;
        inputbox : A_Input_Box;
        label    : A_Label;
    begin
        Dialog(this.all).Construct( view, id, "Resize...", "" );
        this.Set_Layout_Center( 150, 120 );

        label := Create_Label( view, this.id & ":lblDim", "New Dimensions:" );
        label.Set_Layout_LTRB( 5, 5, 5, -20 );
        this.Add( A_Widget(label) );

        label := Create_Label( view, this.id & ":lblWidth", "Width" );
        label.Set_Layout_LTRB( 5, 23, 60, -38 );
        label.Set_Align( Align_Right );
        this.Add( A_Widget(label) );

        inputbox := Create_Input_Box( view, this.id & ":ibWidth" );
        inputbox.Set_Layout_LTRB( -55, 23, 5, -38 );
        inputbox.Set_Constraint( Constraints.Int_Only'Access );
        inputbox.Set_Max_Length( 4 );
        inputbox.Set_Prev( this.id & ":btnCancel" );
        inputbox.Set_Next( this.id & ":ibHeight" );
        this.Add( A_Widget(inputbox) );

        label := Create_Label( view, this.id & ":lblHeight", "Height" );
        label.Set_Layout_LTRB( 5, 41, 60, -56 );
        label.Set_Align( Align_Right );
        this.Add( A_Widget(label) );

        inputbox := Create_Input_Box( view, this.id & ":ibHeight" );
        inputbox.Set_Layout_LTRB( -55, 41, 5, -56 );
        inputbox.Set_Constraint( Constraints.Int_Only'Access );
        inputbox.Set_Max_Length( 4 );
        inputbox.Set_Prev( this.id & ":ibWidth" );
        inputbox.Set_Next( this.id & ":btnOk" );
        this.Add( A_Widget(inputbox) );

        button := Create_Push_Button( view, this.id & ":btnOk", "OK" );
        button.Set_Layout_LTWH( -63, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":ibHeight" );
        button.Set_Next( this.id & ":btnCancel" );
        this.Add( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( -5, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnOk" );
        button.Set_Next( this.id & ":ibWidth" );
        this.Add( A_Widget(button) );

        this.Get_View.Get_Corral.Add_Listener( A_Event_Listener(this), "New_World" );
    exception
        when others =>
            Delete( A_Widget(button) );
            Delete( A_Widget(inputbox) );
            Delete( A_Widget(label) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Resize_Dialog ) is
    begin
        this.Get_View.Get_Corral.Remove_Listener( A_Event_Listener(this), "New_World" );
        Dialog(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    not overriding
    procedure Do_Ok( this : access Resize_Dialog ) is
        view   : constant A_Game_View := A_Game_View(this.Get_View);
        wstr   : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibWidth" )).Get_Text;
        hstr   : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibHeight" )).Get_Text;
        width  : Integer;
        height : Integer;
    begin
        if wstr'Length > 0 and then hstr'Length > 0 then
            width := Integer'Value( wstr );
            height := Integer'Value( hstr );
            begin
                if width > 0 and then height > 0 then
                    Trigger_Resize_World( width, height );
                    this.Hide;
                else
                    raise Constraint_Error with "Invalid map size";
                end if;
            exception
                when e : others =>
                    null;
                    pragma Debug( Dbg( "Error resizing world: " &
                                       Exception_Message( e ), D_VIEW, ERROR ) );
            end;
        end if;
    end Do_Ok;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Resize_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnOk" ) then
                this.Do_Ok;
            elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
                this.Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Resize_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = KEY_ESC and then action.Get_Id = Key_Press then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Resize_Dialog;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
        pragma Unreferenced( resp );
    begin
        if evt.Get_Id = NEW_WORLD_ID then
            this.mapWidth := A_New_World_Event(evt).Get_Width;
            this.mapHeight := A_New_World_Event(evt).Get_Height;
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Show( this : access Resize_Dialog ) is
        view : constant A_Game_View := A_Game_View(this.Get_View);
    begin
        A_Input_Box(view.Get_Widget( this.id & ":ibWidth" )).Set_Text( Image( this.mapWidth ) );
        A_Input_Box(view.Get_Widget( this.id & ":ibHeight" )).Set_Text( Image( this.mapHeight ) );
    end Handle_Show;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access Resize_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_Window.Give_Focus( this.Get_View.Get_Widget( this.id & ":ibWidth" ) );
    end Show;


end Widgets.Containers.Panels.Dialogs.Resize_World;
