
with Ada.Tags;                          use Ada.Tags;
with Entities.Items;
with Entities.Sprites.Keen4;            use Entities.Sprites.Keen4;
with Events.Audio;                      use Events.Audio;
with Events.Entities;                   use Events.Entities;
with Events.Keen;                       use Events.Keen;
with Resources;                         use Resources;

pragma Elaborate_All( Entities.Items );

package body Entities.Items.Keen4 is

    ANM_DELAY    : constant Time_Span := Milliseconds( 115 );
    POINTS_SPEED : constant Float := -40.0;

    --==========================================================================

    function Create_Blue_Gem return A_Entity is
        this : A_Entity := new Blue_Gem;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Blue_Gem;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Blue_Gem ) is
    begin
        Item(this.all).Construct( 15, 12, "items", "gemblue1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Blue_Gem ) is
    begin
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Collect_Gem'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_gem.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Blue_Gem is
        this : aliased Blue_Gem;
    begin
        this.Construct;
        Blue_Gem'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Blue_Gem ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Blue_Gem ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Candybar return A_Entity is
        this : A_Entity := new Candybar;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Candybar;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Candybar ) is
    begin
        Item(this.all).Construct( 16, 16, "items", "candybar1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Candybar ) is
    begin
        Queue_Give_Points( 500 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Points_500'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_points.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Candybar is
        this : aliased Candybar;
    begin
        this.Construct;
        Candybar'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Candybar ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Candybar ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Donut return A_Entity is
        this : A_Entity := new Donut;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Donut;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Donut ) is
    begin
        Item(this.all).Construct( 15, 15, "items", "donut1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Donut ) is
    begin
        Queue_Give_Points( 2000 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Points_2000'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_points.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Donut is
        this : aliased Donut;
    begin
        this.Construct;
        Donut'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Donut ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Donut ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Drop return A_Entity is
        this : A_Entity := new Drop;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Drop;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Drop ) is
    begin
        Item(this.all).Construct( 16, 16, "items", "drop1.bmp", 4, ANM_DELAY );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Drop ) is
    begin
        Queue_Give_Drops( 1 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Collect_Drop'Tag ), this.x, this.y );
        Queue_Play_Sound( "get_drop.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Drop is
        this : aliased Drop;
    begin
        this.Construct;
        Drop'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Drop ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Drop ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Flask return A_Entity is
        this : A_Entity := new Flask;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Flask;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Flask ) is
    begin
        Item(this.all).Construct( 11, 16, "items", "flask1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Flask ) is
    begin
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Yellow_1UP'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_1up.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Flask is
        this : aliased Flask;
    begin
        this.Construct;
        Flask'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Flask ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Flask ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Green_Gem return A_Entity is
        this : A_Entity := new Green_Gem;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Green_Gem;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Green_Gem ) is
    begin
        Item(this.all).Construct( 15, 12, "items", "gemgreen1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Green_Gem ) is
    begin
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Collect_Gem'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_gem.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Green_Gem is
        this : aliased Green_Gem;
    begin
        this.Construct;
        Green_Gem'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Green_Gem ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Green_Gem ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Gum return A_Entity is
        this : A_Entity := new Gum;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Gum;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Gum ) is
    begin
        Item(this.all).Construct( 16, 16, "items", "gum1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Gum ) is
    begin
        Queue_Give_Points( 200 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Points_200'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_points.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Gum is
        this : aliased Gum;
    begin
        this.Construct;
        Gum'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Gum ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Gum ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Icecream return A_Entity is
        this : A_Entity := new Icecream;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Icecream;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Icecream ) is
    begin
        Item(this.all).Construct( 10, 16, "items", "icecream1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Icecream ) is
    begin
        Queue_Give_Points( 5000 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Points_5000'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_points.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Icecream is
        this : aliased Icecream;
    begin
        this.Construct;
        Icecream'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Icecream ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Icecream ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Jawbreaker return A_Entity is
        this : A_Entity := new Jawbreaker;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Jawbreaker;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Jawbreaker ) is
    begin
        Item(this.all).Construct( 15, 15, "items", "jawbreaker1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Jawbreaker ) is
    begin
        Queue_Give_Points( 1000 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Points_1000'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_points.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Jawbreaker is
        this : aliased Jawbreaker;
    begin
        this.Construct;
        Jawbreaker'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Jawbreaker ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Jawbreaker ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Red_Gem return A_Entity is
        this : A_Entity := new Red_Gem;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Red_Gem;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Red_Gem ) is
    begin
        Item(this.all).Construct( 15, 12, "items", "gemred1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Red_Gem ) is
    begin
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Collect_Gem'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_gem.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Red_Gem is
        this : aliased Red_Gem;
    begin
        this.Construct;
        Red_Gem'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Red_Gem ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Red_Gem ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Soda return A_Entity is
        this : A_Entity := new Soda;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Soda;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Soda ) is
    begin
        Item(this.all).Construct( 10, 16, "items", "soda1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Soda ) is
    begin
        Queue_Give_Points( 100 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Points_100'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_points.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Soda is
        this : aliased Soda;
    begin
        this.Construct;
        Soda'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Soda ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Soda ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Stunner return A_Entity is
        this : A_Entity := new Stunner;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Stunner;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Stunner ) is
    begin
        Item(this.all).Construct( 15, 11, "items", "gun1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Stunner ) is
    begin
        Queue_Give_Ammo( 5 );
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Collect_Stunner'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_gun.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Stunner is
        this : aliased Stunner;
    begin
        this.Construct;
        Stunner'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Stunner ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Stunner ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Yellow_Gem return A_Entity is
        this : A_Entity := new Yellow_Gem;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Yellow_Gem;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Yellow_Gem ) is
    begin
        Item(this.all).Construct( 15, 12, "items", "gemyellow1.bmp", 2, ANM_DELAY * 4 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Give_Item( this : access Yellow_Gem ) is
    begin
        Queue_Delete_Entity( this.id );
        Queue_Spawn_Entity( External_Tag( Collect_Gem'Tag ),
                            this.x, this.y, yv => POINTS_SPEED );
        Queue_Play_Sound( "get_gem.wav" );
    end Give_Item;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Yellow_Gem is
        this : aliased Yellow_Gem;
    begin
        this.Construct;
        Yellow_Gem'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Yellow_Gem ) is
    begin
        Item'Read( stream, Item(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Yellow_Gem ) is
    begin
        Item'Write( stream, Item(obj) );
    end Object_Write;

begin

    Entities.Factory.Register_Class( External_Tag( Blue_Gem'Tag ),   Create_Blue_Gem'Access );
    Entities.Factory.Register_Class( External_Tag( Candybar'Tag ),   Create_Candybar'Access );
    Entities.Factory.Register_Class( External_Tag( Donut'Tag ),      Create_Donut'Access );
    Entities.Factory.Register_Class( External_Tag( Drop'Tag ),       Create_Drop'Access );
    Entities.Factory.Register_Class( External_Tag( Flask'Tag ),      Create_Flask'Access );
    Entities.Factory.Register_Class( External_Tag( Green_Gem'Tag ),  Create_Green_Gem'Access );
    Entities.Factory.Register_Class( External_Tag( Gum'Tag ),        Create_Gum'Access );
    Entities.Factory.Register_Class( External_Tag( Icecream'Tag ),   Create_Icecream'Access );
    Entities.Factory.Register_Class( External_Tag( Jawbreaker'Tag ), Create_Jawbreaker'Access );
    Entities.Factory.Register_Class( External_Tag( Red_Gem'Tag ),    Create_Red_Gem'Access );
    Entities.Factory.Register_Class( External_Tag( Soda'Tag ),       Create_Soda'Access );
    Entities.Factory.Register_Class( External_Tag( Stunner'Tag ),    Create_Stunner'Access );
    Entities.Factory.Register_Class( External_Tag( Yellow_Gem'Tag ), Create_Yellow_Gem'Access );

end Entities.Items.Keen4;
