
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
--with Debugging;                         use Debugging;
with Events.Entities;                   use Events.Entities;
with Entities.Players;
with Preferences;

pragma Elaborate_All( Entities.Players );
pragma Elaborate_All( Preferences );

package body Entities.Players.Little_Keen is

    PREF_PREFIX : constant String := "entity.player.little_keen.";

    IDLE_DELAY    : constant Time_Span := Milliseconds( 4500 );
    IDLE_DURATION : constant Time_Span := Milliseconds( 1800 );

    ----------------------------------------------------------------------------

    type Frame_Type is
        record
            filename : String_Access := null;
            id       : Integer := -1;
        end record;

    type Frame_Array is array(Integer range <>) of Frame_Type;

    type Direction_Array is array (Direction_8) of access Frame_Array;

    type Animation( directional : Boolean ) is
        record
            looping    : Boolean;
            frameDelay : Time_Span;
            case directional is
                when True =>
                    dir : access Direction_Array := null;
                when False =>
                    frames : access Frame_Array := null;
            end case;
        end record;

    ----------------------------------------------------------------------------

    type Visible_Action is (
        Standing,
        Walking,
        Swimming,
        Waving_Bored
    );

    ----------------------------------------------------------------------------

    type Action_Frames is array (Visible_Action) of access Animation;

    frames : constant Action_Frames := Action_Frames'(
        Standing => new Animation'(
            directional => True,
            looping     => True,
            frameDelay  => Milliseconds( 115 ),
            dir         => new Direction_Array'(
                D8_Left       => new Frame_Array'(1 => (filename => new String'("map_walk_left1.bmp" ), others => <>)),
                D8_Right      => new Frame_Array'(1 => (filename => new String'("map_walk_right1.bmp" ), others => <>)),
                D8_Up         => new Frame_Array'(1 => (filename => new String'("map_walk_up1.bmp" ), others => <>)),
                D8_Down       => new Frame_Array'(1 => (filename => new String'("map_walk_down1.bmp" ), others => <>)),
                D8_Up_Left    => new Frame_Array'(1 => (filename => new String'("map_walk_up_left1.bmp" ), others => <>)),
                D8_Up_Right   => new Frame_Array'(1 => (filename => new String'("map_walk_up_right1.bmp" ), others => <>)),
                D8_Down_Left  => new Frame_Array'(1 => (filename => new String'("map_walk_down_left1.bmp" ), others => <>)),
                D8_Down_Right => new Frame_Array'(1 => (filename => new String'("map_walk_down_right1.bmp" ), others => <>)))),
        Walking => new Animation'(
            directional => True,
            looping     => True,
            frameDelay  => Milliseconds( 115 ),
            dir         => new Direction_Array'(
                D8_Left => new Frame_Array'(
                    (filename => new String'("map_walk_left1.bmp" ), others => <>),
                    (filename => new String'("map_walk_left2.bmp" ), others => <>),
                    (filename => new String'("map_walk_left1.bmp" ), others => <>),
                    (filename => new String'("map_walk_left3.bmp" ), others => <>)),
                D8_Right => new Frame_Array'(
                    (filename => new String'("map_walk_right1.bmp" ), others => <>),
                    (filename => new String'("map_walk_right2.bmp" ), others => <>),
                    (filename => new String'("map_walk_right1.bmp" ), others => <>),
                    (filename => new String'("map_walk_right3.bmp" ), others => <>)),
                D8_Up => new Frame_Array'(
                    (filename => new String'("map_walk_up1.bmp" ), others => <>),
                    (filename => new String'("map_walk_up2.bmp" ), others => <>),
                    (filename => new String'("map_walk_up1.bmp" ), others => <>),
                    (filename => new String'("map_walk_up3.bmp" ), others => <>)),
                D8_Down => new Frame_Array'(
                    (filename => new String'("map_walk_down1.bmp" ), others => <>),
                    (filename => new String'("map_walk_down2.bmp" ), others => <>),
                    (filename => new String'("map_walk_down1.bmp" ), others => <>),
                    (filename => new String'("map_walk_down3.bmp" ), others => <>)),
                D8_Up_Left => new Frame_Array'(
                    (filename => new String'("map_walk_up_left1.bmp" ), others => <>),
                    (filename => new String'("map_walk_up_left2.bmp" ), others => <>),
                    (filename => new String'("map_walk_up_left1.bmp" ), others => <>),
                    (filename => new String'("map_walk_up_left3.bmp" ), others => <>)),
                D8_Up_Right => new Frame_Array'(
                    (filename => new String'("map_walk_up_right1.bmp" ), others => <>),
                    (filename => new String'("map_walk_up_right2.bmp" ), others => <>),
                    (filename => new String'("map_walk_up_right1.bmp" ), others => <>),
                    (filename => new String'("map_walk_up_right3.bmp" ), others => <>)),
                D8_Down_Left => new Frame_Array'(
                    (filename => new String'("map_walk_down_left1.bmp" ), others => <>),
                    (filename => new String'("map_walk_down_left2.bmp" ), others => <>),
                    (filename => new String'("map_walk_down_left1.bmp" ), others => <>),
                    (filename => new String'("map_walk_down_left3.bmp" ), others => <>)),
                D8_Down_Right => new Frame_Array'(
                    (filename => new String'("map_walk_down_right1.bmp" ), others => <>),
                    (filename => new String'("map_walk_down_right2.bmp" ), others => <>),
                    (filename => new String'("map_walk_down_right1.bmp" ), others => <>),
                    (filename => new String'("map_walk_down_right3.bmp" ), others => <>)))),
        Swimming => new Animation'(
            directional => True,
            looping     => True,
            frameDelay  => Milliseconds( 115 ),
            dir         => new Direction_Array'(
                D8_Left => new Frame_Array'(
                    (filename => new String'("map_swim_left1.bmp" ), others => <>),
                    (filename => new String'("map_swim_left2.bmp" ), others => <>),
                    (filename => new String'("map_swim_left1.bmp" ), others => <>),
                    (filename => new String'("map_swim_left3.bmp" ), others => <>)),
                D8_Right => new Frame_Array'(
                    (filename => new String'("map_swim_right1.bmp" ), others => <>),
                    (filename => new String'("map_swim_right2.bmp" ), others => <>),
                    (filename => new String'("map_swim_right1.bmp" ), others => <>),
                    (filename => new String'("map_swim_right3.bmp" ), others => <>)),
                D8_Up => new Frame_Array'(
                    (filename => new String'("map_swim_up1.bmp" ), others => <>),
                    (filename => new String'("map_swim_up2.bmp" ), others => <>),
                    (filename => new String'("map_swim_up1.bmp" ), others => <>),
                    (filename => new String'("map_swim_up3.bmp" ), others => <>)),
                D8_Down => new Frame_Array'(
                    (filename => new String'("map_swim_down1.bmp" ), others => <>),
                    (filename => new String'("map_swim_down2.bmp" ), others => <>),
                    (filename => new String'("map_swim_down1.bmp" ), others => <>),
                    (filename => new String'("map_swim_down3.bmp" ), others => <>)),
                D8_Up_Left => new Frame_Array'(
                    (filename => new String'("map_swim_up_left1.bmp" ), others => <>),
                    (filename => new String'("map_swim_up_left2.bmp" ), others => <>),
                    (filename => new String'("map_swim_up_left1.bmp" ), others => <>),
                    (filename => new String'("map_swim_up_left3.bmp" ), others => <>)),
                D8_Up_Right => new Frame_Array'(
                    (filename => new String'("map_swim_up_right1.bmp" ), others => <>),
                    (filename => new String'("map_swim_up_right2.bmp" ), others => <>),
                    (filename => new String'("map_swim_up_right1.bmp" ), others => <>),
                    (filename => new String'("map_swim_up_right3.bmp" ), others => <>)),
                D8_Down_Left => new Frame_Array'(
                    (filename => new String'("map_swim_down_left1.bmp" ), others => <>),
                    (filename => new String'("map_swim_down_left2.bmp" ), others => <>),
                    (filename => new String'("map_swim_down_left1.bmp" ), others => <>),
                    (filename => new String'("map_swim_down_left3.bmp" ), others => <>)),
                D8_Down_Right => new Frame_Array'(
                    (filename => new String'("map_swim_down_right1.bmp" ), others => <>),
                    (filename => new String'("map_swim_down_right2.bmp" ), others => <>),
                    (filename => new String'("map_swim_down_right1.bmp" ), others => <>),
                    (filename => new String'("map_swim_down_right3.bmp" ), others => <>)))),
        Waving_Bored => new Animation'(
            directional => False,
            looping     => True,
            frameDelay  => IDLE_DURATION / 5,
            frames      => new Frame_Array'(
                (filename => new String'("map_wave1.bmp" ), others => <>),
                (filename => new String'("map_wave2.bmp" ), others => <>)))
    );

    ----------------------------------------------------------------------------

    function Get_Pref( shortName : String ) return Integer is
    begin
        return Preferences.Get_Pref( "development", PREF_PREFIX & shortName );
    end Get_Pref;
    pragma Inline( Get_Pref );

    ----------------------------------------------------------------------------

    function Get_Pref( shortName : String ) return Float is
    begin
        return Preferences.Get_Pref( "development", PREF_PREFIX & shortName );
    end Get_Pref;
    pragma Inline( Get_Pref );

    ----------------------------------------------------------------------------

    function Get_Pref( shortName : String ) return String is
    begin
        return Preferences.Get_Pref( "development", PREF_PREFIX & shortName );
    end Get_Pref;
    pragma Inline( Get_Pref );

    --==========================================================================

    function Create_Player return A_Entity is
        this : constant A_Entity := new Little_Keen;
    begin
        this.Construct;
        return this;
    end Create_Player;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Little_Keen ) is
    begin
        Player(this.all).Construct( Get_Pref( "width" ),
                                    Get_Pref( "height" ),
                                    Get_Pref( "library" ) );
        this.walkSpeed := Get_Pref( "walk_speed" );
        this.walkAcceleration := Get_Pref( "walk_acceleration" );
        this.Update_Frame( notify => False );
    end Construct;

    ----------------------------------------------------------------------------

    procedure Die( this : access Little_Keen ) is
        pragma Unreferenced( this );
    begin
        null;
    end Die;

    ----------------------------------------------------------------------------

    overriding
    procedure Face( this : access Little_Keen; dir : Direction_Type ) is
    begin
        this.dir := dir;
        this.Update_Frame;
    end Face;

    ----------------------------------------------------------------------------

    function Is_Busy( this : not null access Little_Keen'Class ) return Boolean is
    begin
        if this.actionDuration > Time_Span_Zero then
            return this.actionStart + this.actionDuration > this.age;
        end if;
        return False;
    end Is_Busy;

    ----------------------------------------------------------------------------

    overriding
    procedure Move_Start( this : access Little_Keen; dir : Cardinal_Direction ) is
    begin
        if not this.moving(dir) and then not this.moving(Opposite( dir )) then
            if not this.moving then
                this.dir := To_Direction( dir );
            else
                this.dir := this.dir + dir;
            end if;
            this.moving(dir) := True;
            if this.action = Act_Idle then
                this.Start_Action( Act_Walking );
            end if;
            Queue_Accelerate( this.id, dir, this.walkSpeed, this.walkAcceleration );
            this.Update_Frame;
        end if;
    end Move_Start;

    ----------------------------------------------------------------------------

    overriding
    procedure Move_Stop( this : access Little_Keen; dir : Cardinal_Direction ) is
    begin
        if this.moving(dir) then
            this.moving(dir) := False;
            this.dir := this.dir - dir;
            Queue_Accelerate( this.id, dir, 0.0, 0.0 );
            this.Update_Frame;
        end if;
    end Move_Stop;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Little_Keen is
        this : aliased Little_Keen;
    begin
        this.Construct;
        Little_Keen'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Little_Keen ) is
    begin
        Player'Read( stream, Player(obj) );
        obj.action := Action_Type'Input( stream );
        obj.actionStart := Time_Span'Input( stream );
        obj.actionDuration := Time_Span'Input( stream );
        obj.idleStart := Time_Span'Input( stream );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Little_Keen ) is
    begin
        Player'Write( stream, Player(obj) );
        Action_Type'Output( stream, obj.action );
        Time_Span'Output( stream, obj.actionStart );
        Time_Span'Output( stream, obj.actionDuration );
        Time_Span'Output( stream, obj.idleStart );
    end Object_Write;

    ----------------------------------------------------------------------------

    procedure Reset_Idle( this : not null access Little_Keen'Class ) is
    begin
        this.idleStart := this.age;
    end Reset_Idle;

    ----------------------------------------------------------------------------

    procedure Start_Action( this   : not null access Little_Keen'Class;
                            action : Action_Type;
                            length : Time_Span := Time_Span_Zero ) is
    begin
        this.action := action;
        this.actionStart := this.age;
        this.actionDuration := length;
    end Start_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Tick( this : access Little_Keen; upTime, dt : Time_Span ) is
    begin
        Player(this.all).Tick( upTime, dt );

        case this.action is
            when Act_Walking =>
                if not this.moving then
                    -- check if its time to start an idle animation
                    if this.age >= this.idleStart + IDLE_DELAY then
                        this.Start_Action( Act_Idle, IDLE_DURATION );
                    end if;
                else
                    this.Reset_Idle;
                end if;

            when Act_Swimming =>
                this.Reset_Idle;

            when Act_Idle =>
                -- if keen is done waving then go back to moving
                if not this.Is_Busy then
                    this.Start_Action( Act_Walking );
                end if;
                this.Reset_Idle;

        end case;

        this.upTime := upTime;
        this.Update_Frame;
    end Tick;

    ----------------------------------------------------------------------------

    overriding
    procedure Update_Frame( this : access Little_Keen; notify : Boolean := True ) is

        ------------------------------------------------------------------------

        function Get_Frame_Id( action   : Visible_Action;
                               dir      : Direction_8 := D8_Up;
                               reversed : Boolean := False ) return Natural is

            --------------------------------------------------------------------

            procedure Resolve_Id( id : out Natural; frame : in out Frame_Type ) is
            begin
                if frame.id < 0 then
                    frame.id := this.lib.Get_Id( frame.filename.all );
                end if;
                id := frame.id;
            end Resolve_Id;

            --------------------------------------------------------------------

            frameArray : access Frame_Array;
            frameNum   : Natural := 0;
            id         : Natural := 0;
        begin
            if frames(action).directional then
                frameArray := frames(action).dir(dir);
            else
                frameArray := frames(action).frames;
            end if;

            if frameArray'Length = 1 then
                frameNum := 0;
            else
                if frames(action).looping then
                    frameNum := ((this.age - this.actionStart) / frames(action).frameDelay) mod frameArray'Length;
                else
                    frameNum := Natural'Min( ((this.age - this.actionStart) / frames(action).frameDelay), frameArray'Length - 1 );
                end if;
            end if;

            if reversed then
                Resolve_Id( id, frameArray(frameArray'Last - frameNum) );
            else
                Resolve_Id( id, frameArray(frameArray'First + frameNum) );
            end if;

            return id;
        end Get_Frame_Id;

        ------------------------------------------------------------------------

        id : Natural := 0;
    begin
        -- determine keen's current visible action
        case this.action is
            when Act_Walking =>
                if not this.moving then
                    id := Get_Frame_Id( Standing, To_D8( this.dir ) );
                else
                    id := Get_Frame_Id( Walking, To_D8( this.dir ) );
                end if;

            when Act_Swimming =>
                id := Get_Frame_Id( Swimming, To_D8( this.dir ) );

            when Act_Idle =>
                id := Get_Frame_Id( Waving_Bored );

        end case;

        this.Set_Frame( id, notify );
    end Update_Frame;

begin

    Entities.Factory.Register_Class( "player.Map", Create_Player'Access );

    Preferences.Set_Default( "development", "entity.player.little_keen.library",           "keen" );
    Preferences.Set_Default( "development", "entity.player.little_keen.height",            16     );
    Preferences.Set_Default( "development", "entity.player.little_keen.width",             16     );
    Preferences.Set_Default( "development", "entity.player.little_keen.walk_speed",        92.0   ); -- measured at 80 px/s with keen4e
    Preferences.Set_Default( "development", "entity.player.little_keen.walk_acceleration", 700.0  );

end Entities.Players.Little_Keen;
