
with Ada.Tags;                          use Ada.Tags;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Entities.Players;                  use Entities.Players;
with Entities.Players.Keen;             use Entities.Players.Keen;
with Entities.Triggers;
with Events.World;                      use Events.World;
with Resources;                         use Resources;
with Ada.Strings;                       use Ada.Strings;
with Ada.Strings.Fixed;                 use Ada.Strings.Fixed;

pragma Elaborate_All( Entities.Triggers );

package body Entities.Triggers.Keen is

    ICON_FILENAME : constant String := "trigger_entity.bmp";

    ----------------------------------------------------------------------------

    function Create_Keen_Trigger return A_Entity is
        this : A_Entity := new Keen_Trigger;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Keen_Trigger;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Keen_Trigger ) is
    begin
        Trigger(this.all).Construct( "icons", ICON_FILENAME );
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    procedure Do_Enterlevel( this : access Keen_Trigger; levelName : String ) is
    begin
        Queue_Load_World( levelName );
        this.activatable := False;
    end Do_Enterlevel;

    ----------------------------------------------------------------------------

    not overriding
    procedure Do_Teleport( this      : access Keen_Trigger;
                           arg       : String;
                           activator : not null A_Entity ) is

        ------------------------------------------------------------------------

        procedure Parse_XY( coords : String; x, y : out Float ) is
            comma : constant Integer := Index( coords, "," );
        begin
            if comma > coords'First and then comma < coords'Last then
                x := Float'Value( Trim( coords(coords'First..comma-1), Both ) );
                y := Float'Value( Trim( coords(comma+1..coords'Last), Both ) );
            else
                x := this.x;
                y := this.y;
            end if;
        exception
            when others =>
                x := this.x;
                y := this.y;
        end Parse_XY;

        ------------------------------------------------------------------------

        x, y : Float;
    begin
        if activator.all in Players.Keen.Keen'Class then
            Parse_XY( arg, x, y );
            Players.Keen.Keen(activator.all).Teleport( this.x, this.y, x, y );
        end if;
    end Do_Teleport;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Keen_Trigger is
        this : aliased Keen_Trigger;
    begin
        this.Construct;
        Keen_Trigger'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Keen_Trigger ) is
    begin
        Trigger'Read( stream, Trigger(obj) );
        obj.frame := obj.lib.Get_Id( ICON_FILENAME );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Keen_Trigger ) is
    begin
        Trigger'Write( stream, Trigger(obj) );
    end Object_Write;

    ----------------------------------------------------------------------------

    overriding
    procedure Execute( this      : access Keen_Trigger;
                       action,
                       argument  : String;
                       activator : A_Entity ) is
    begin
        pragma Debug( Dbg( To_String( this ) & ".Execute: " & action &
                           "( " & argument & " )", D_ENTITIES, INFO ) );

        if action = "enterlevel" then
            this.Do_Enterlevel( argument );

        elsif action = "teleport" then
            this.Do_Teleport( argument, activator );

        else
            null;
            pragma Debug( action'Length > 0,
                          Dbg( "Trigger: Ignored unrecognized action '" &
                               action & "'", D_ENTITIES, WARNING ) );
            pragma Debug( action'Length = 0,
                          Dbg( "Trigger: Missing action", D_ENTITIES, WARNING ) );
        end if;
    end Execute;

begin

    Entities.Factory.Register_Class( External_Tag( Keen_Trigger'Tag ), Create_Keen_Trigger'Access );

end Entities.Triggers.Keen;
