
with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Tiles;

pragma Elaborate_All( Tiles );

package body Tiles.Keen is

    SYS_VERSION    : constant := 4;
    SYS_IDENTIFIER : constant String := "keen7";

    ----------------------------------------------------------------------------

    function ">"( l, r : Action_Type ) return Boolean is
    begin
        if l = r then
            return False;
        elsif r = NONE then
            return True;
        else
            return Action_Type'Pos( l ) < Action_Type'Pos( r );
        end if;
    end ">";

    --==========================================================================

    function Create_Tile return A_Tile is
        this : constant A_Tile := new Keen_Tile;
    begin
        this.Construct;
        return this;
    end Create_Tile;

    ----------------------------------------------------------------------------

    overriding
    procedure Set_Attribute( this  : in out Keen_Tile;
                             found : out Boolean;
                             name  : String;
                             val   : String := "" ) is

        ------------------------------------------------------------------------

        procedure Parse_Action_Type( str     : String;
                                     action  : in out Action_Type;
                                     success : out Boolean ) is
            lower_str : constant String := To_Lower( str );
        begin
            success := False;
            for i in Action_Type'Range loop
                if To_Lower( Action_Type'Image( i ) ) = lower_str then
                    action := i;
                    success := True;
                    exit;
                end if;
            end loop;
        end Parse_Action_Type;

        ------------------------------------------------------------------------

        success : Boolean;
    begin
        Tile_Object(this).Set_Attribute( found, name, val );
        if not found then
            found := True;

            -- action=<Tiles.Keen.Action_Type enum values>
            if name = "action" then
                Parse_Action_Type( val, this.action, success );
                if not success then
                    raise ATTRIBUTE_ERROR with "Invalid value for 'action'";
                end if;
            else
                found := False;
            end if;
        end if;
    end Set_Attribute;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Action( this : access Keen_Tile ) return Action_Type is
    begin
        return this.action;
    end Get_Action;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Keen_Tile is
        this : aliased Keen_Tile;
    begin
        this.Construct;
        Keen_Tile'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Output( stream : access Root_Stream_Type'Class; obj : Keen_Tile ) is
    begin
        Keen_Tile'Write( stream, obj );
    end Object_Output;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Keen_Tile ) is
    begin
        Tile_Object'Read( stream, Tile_Object(obj) );
        obj.action := Action_Type'Input( stream );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Keen_Tile ) is
    begin
        Tile_Object'Write( stream, Tile_Object(obj) );
        Action_Type'Output( stream, obj.action );
    end Object_Write;

begin

    Register_Version( SYS_VERSION );
    Register_Identifier( SYS_IDENTIFIER );
    Register_Allocator( Create_Tile'Access );

end Tiles.Keen;
